# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-03 Code cleanup
#             Cornelius Kölbel <cornelius@privacyidea.org>
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file contains the definition of the TOTP token class
  
  Dependencies: -

'''

import logging
import time
import math
import datetime

import traceback

from privacyidea.lib.HMAC import HmacOtp
from privacyidea.lib.util import getParam
from privacyidea.lib.util import generate_otpkey
from privacyidea.lib.config import getFromConfig
from privacyidea.lib.log import log_with
from privacyidea.lib.tokenclass import TokenClass
from privacyidea.lib.tokens.hmactoken import HmacTokenClass

optional = True
required = False

keylen = {'sha1': 20,
          'sha256': 32,
          'sha512': 64
          }

log = logging.getLogger(__name__)


###############################################
"""
TOTP Algorithm

   This variant of the HOTP algorithm specifies the calculation of a
   one-time password value, based on a representation of the counter as
   a time factor.

4.1.  Notations

   - X represents the time step in seconds (default value X = 30
   seconds) and is a system parameter;

   - T0 is the Unix time to start counting time steps (default value is
   0, Unix epoch) and is also a system parameter.

4.2.  Description

   Basically, we define TOTP as TOTP = HOTP(K, T) where T is an integer
   and represents the number of time steps between the initial counter
   time T0 and the current Unix time (i.e. the number of seconds elapsed
   since midnight UTC of January 1, 1970).

   More specifically T = (Current Unix time - T0) / X where:

   - X represents the time step in seconds (default value X = 30
   seconds) and is a system parameter;

   - T0 is the Unix time to start counting time steps (default value is
   0, Unix epoch) and is also a system parameter;

   - The default floor function is used in the computation.  For
   example, with T0 = 0 and time step X = 30, T = 1 if the current Unix
   time is 59 seconds and T = 2 if the current Unix time is 60 seconds.

M'Raihi, et al.          Expires March 12, 2011                 [Page 5]

Internet-Draft                HOTPTimeBased               September 2010


"""


###############################################
class TimeHmacTokenClass(HmacTokenClass):

    resyncDiffLimit = 3

    @log_with(log)
    def __init__(self, aToken):
        '''
        constructor - create a token object

        :param aToken: instance of the orm db object
        :type aToken:  orm object

        '''
        TokenClass.__init__(self, aToken)
        self.setType(u"TOTP")
        self.hKeyRequired = True

        ''' timeStep defines the granularity: '''
        self.timeStep = getFromConfig("totp.timeStep", 30)

        ''' window size in seconds:
            30 seconds with as step width of 30 seconds results
            in a window of 1  which is one attempt
        '''
        self.timeWindow = getFromConfig("totp.timeWindow", 180)

        '''the time shift is specified in seconds  - and could be
        positive and negative
        '''
        self.timeShift = getFromConfig("totp.timeShift", 0)

        '''we support various hashlib methods, but only on create
        which is effectively set in the update
        '''
        self.hashlibStr = getFromConfig("totp.hashlib", u'sha1')
        return
    
    @classmethod
    def getClassType(cls):
        '''
        getClassType - return the token type shortname

        :return: 'totp'
        :rtype: string

        '''
        return "totp"

    @classmethod
    def getClassPrefix(cls):
        return "TOTP"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        '''
        res = {'type': 'totp',
               'title': 'HMAC Time Token',
               'description': ('time based otp token using '
                               'the TOTP algorithm'),
               'init': {'page': {'html': 'totptoken.mako',
                                 'scope': 'enroll', },
                        'title': {'html': 'totptoken.mako',
                                  'scope': 'enroll.title', },
                        },
               'config': {'page': {'html': 'totptoken.mako',
                                   'scope': 'config', },
                          'title': {'html': 'totptoken.mako',
                                    'scope': 'config.title'},
                          },
               'selfservice': {'enroll': {'page': {'html': 'totptoken.mako',
                                                   'scope': 'selfservice.'
                                                   'enroll'},
                                          'title': {'html': 'totptoken.mako',
                                                    'scope': 'selfservice.'
                                                    'title.enroll'},
                                          },
                               },
               'policy': {'selfservice': {'totp_timestep': {'type': 'int',
                                                            'value': [30, 60],
                                                            'desc': 'Specify '
                                                            'the time step of '
                                                            'the timebased OTP'
                                                            ' token.'
                                                            },
                                          'totp_hashlib': {'type': 'int',
                                                           'value': [1, 2],
                                                           'desc': 'Specify '
                                                           'the hashlib to be '
                                                           'used. Can be sha1'
                                                           ' (1) or sha2-256 '
                                                           '(2).'
                                                           },
                                          },
                          },
               }

        if key is not None and key in res:
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret

    @log_with(log)
    def update(self, param):
        '''
        update - process the initialization parameters

        :param param: dict of initialization parameters
        :type param: dict

        :return: nothing
        '''
        # check for the required parameters
        val = getParam(param, "hashlib", optional)
        if val is not None:
            self.hashlibStr = val
        else:
            self.hashlibStr = 'sha1'

        otpKey = ''

        if (self.hKeyRequired is True):
            genkey = int(getParam(param, "genkey", optional) or 0)
            if 1 == genkey:
                # if hashlibStr not in keylen dict, this will
                # raise an Exception
                otpKey = generate_otpkey(keylen.get(self.hashlibStr))
                del param['genkey']
            else:
                # genkey not set: check otpkey is given
                # this will raise an exception if otpkey is not present
                otpKey = getParam(param, "otpkey", required)

        # finally set the values for the update
        param['otpkey'] = otpKey
        param['hashlib'] = self.hashlibStr

        val = getParam(param, "otplen", optional)
        if val is not None:
            self.setOtpLen(int(val))
        else:
            self.setOtpLen(getFromConfig("DefaultOtpLen"))

        val = getParam(param, "timeStep", optional)
        if val is not None:
            self.timeStep = val

        val = getParam(param, "timeWindow", optional)
        if val is not None:
            self.timeWindow = val

        val = getParam(param, "timeShift", optional)
        if val is not None:
            self.timeShift = val

        HmacTokenClass.update(self, param)

        self.addToTokenInfo("timeWindow", self.timeWindow)
        self.addToTokenInfo("timeShift", self.timeShift)
        self.addToTokenInfo("timeStep", self.timeStep)
        self.addToTokenInfo("hashlib", self.hashlibStr)

        return

    @log_with(log)
    def check_otp_exist(self, otp, window=10):
        '''
        checks if the given OTP value is/are values of this very token.
        This is used to autoassign and to determine the serial number of
        a token.

        :param otp: the to be verified otp value
        :type otp: string

        :param window: the lookahead window for the counter
        :type window: int

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        res = -1

        try:
            counter = int(self.token.privacyIDEACount)
        except ValueError as ex:
            log.warning("a value error occurred while converting: counter %r "
                        ": ValueError: %r ret: %r "
                        % (self.token.privacyIDEACount, ex, res))
            return res

        res = self.checkOtp(otp, counter, range)

        return res

    def _time2counter_(self, T0, timeStepping=60):
        rnd = 0.5
        counter = int((T0 / timeStepping) + rnd)
        return counter

    def _counter2time_(self, counter, timeStepping=60):
        rnd = 0.5
        T0 = (float(counter) - rnd) * timeStepping
        return T0

    def _getTimeFromCounter(self, counter, timeStepping=30, rnd=1):
        idate = int(counter - rnd) * timeStepping
        ddate = datetime.datetime.fromtimestamp(idate / 1.0)
        return ddate

    @log_with(log)
    def time2float(self, curTime):
        '''
        time2float - convert a datetime object or an datetime sting into a
        float
        s. http://bugs.python.org/issue12750

        :param curTime: time in datetime format
        :type curTime: datetime object

        :return: time as float
        :rtype: float
        '''
        dt = datetime.datetime.now()
        if type(curTime) == datetime.datetime:
            dt = curTime
        elif type(curTime) == unicode:
            if '.' in curTime:
                tFormat = "%Y-%m-%d %H:%M:%S.%f"
            else:
                tFormat = "%Y-%m-%d %H:%M:%S"
            try:
                dt = datetime.datetime.strptime(curTime, tFormat)
            except Exception as ex:
                log.error('Error during conversion of datetime: %r' % (ex))
                log.error("%r" % traceback.format_exc())
                raise Exception(ex)
        else:
            log.error("invalid curTime: %s. You need to specify a "
                      "datetime.datetime" % type(curTime))
            raise Exception("[time2float] invalid curTime: %s. You need to "
                            "specify a datetime.datetime" % type(curTime))

        td = (dt - datetime.datetime(1970, 1, 1))
        # for python 2.6 compatibility, we have to implement
        # 2.7 .total_seconds()::
        # TODO: fix to float!!!!
        tCounter = ((td.microseconds +
                     (td.seconds + td.days * 24 * 3600)
                     * 10 ** 6) * 1.0) / 10 ** 6
        return tCounter

    @log_with(log)
    def checkOtp(self, anOtpVal, counter, window, options=None):
        '''
        checkOtp - validate the token otp against a given otpvalue

        :param anOtpVal: the to be verified otpvalue
        @type anOtpVal:  string

        :param counter: the counter state, that should be verified
        :type counter: int

        :param window: the counter +window, which should be checked
        :type window: int

        :param options: the dict, which could contain token specific info
        :type options: dict

        :return: the counter state or -1
        :rtype: int

        '''

        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError as e:
            raise e

        secretHOtp = self.token.getHOtpKey()
        self.hashlibStr = self.getFromTokenInfo("hashlib", self.hashlibStr)

        timeStepping = int(self.getFromTokenInfo("timeStep", self.timeStep))
        window = int(self.getFromTokenInfo("timeWindow", self.timeWindow))
        shift = int(self.getFromTokenInfo("timeShift", self.timeShift))

        # oldCounter we have to remove one, as the normal otp handling will
        # increment
        oCount = self.getOtpCount() - 1

        initTime = -1
        if options is not None and type(options) == dict:
            initTime = int(options.get('initTime', -1))

        if oCount < 0:
            oCount = 0
        log.debug("timestep: %i, timeWindow: %i, timeShift: %i" %
                  (timeStepping, window, shift))
        inow = int(time.time())

        T0 = time.time() + shift
        if initTime != -1:
            T0 = int(initTime)

        log.debug("T0 : %i" % T0)
        counter = self._time2counter_(T0, timeStepping=timeStepping)

        otime = self._getTimeFromCounter(oCount, timeStepping=timeStepping)
        ttime = self._getTimeFromCounter(counter, timeStepping=timeStepping)

        log.debug("last log: %r :: %r" % (oCount, otime))
        log.debug("counter : %r :: %r <==> %r" %
                  (counter, ttime, datetime.datetime.now()))

        log.debug("shift   : %r " % (shift))

        hmac2Otp = HmacOtp(secretHOtp,
                           counter,
                           otplen,
                           self.getHashlib(self.hashlibStr))
        res = hmac2Otp.checkOtp(anOtpVal,
                                int(window / timeStepping),
                                symetric=True)

        log.debug("comparing the result %i to the old counter %i."
                  % (res, oCount))
        if res != -1 and oCount != 0 and res <= oCount:
            if initTime == -1:
                log.warning("a previous OTP value was used again! former "
                            "tokencounter: %i, presented counter %i" %
                            (oCount, res))
                res = -1
                return res

        if -1 == res:
            # autosync: test if two consecutive otps have been provided
            res = self.autosync(hmac2Otp, anOtpVal)

        if res != -1:
            # on success, we have to save the last attempt
            self.setOtpCount(counter)

            #
            # here we calculate the new drift/shift between the server time
            # and the tokentime
            #
            tokentime = self._counter2time_(res, timeStepping)
            tokenDt = datetime.datetime.fromtimestamp(tokentime / 1.0)

            nowDt = datetime.datetime.fromtimestamp(inow / 1.0)

            lastauth = self._counter2time_(oCount, timeStepping)
            lastauthDt = datetime.datetime.fromtimestamp(lastauth / 1.0)

            log.debug("last auth : %r" % (lastauthDt))
            log.debug("tokentime : %r" % (tokenDt))
            log.debug("now       : %r" % (nowDt))
            log.debug("delta     : %r" % (tokentime - inow))

            new_shift = (tokentime - inow)
            log.debug("the counter %r matched. New shift: %r" %
                      (res, new_shift))
            self.addToTokenInfo('timeShift', new_shift)
        return res

    @log_with(log)
    def autosync(self, hmac2Otp, anOtpVal):
        '''
        auto - sync the token based on two otp values
        - internal method to realize the autosync within the
        checkOtp method

        :param hmac2Otp: the hmac object (with reference to the token secret)
        :type hmac2Otp: hmac object

        :param anOtpVal: the actual otp value
        :type anOtpVal: string

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        res = -1
        autosync = False

        try:
            async = getFromConfig("AutoResync")
            if async is None:
                autosync = False
            elif "true" == async.lower():
                autosync = True
            elif "false" == async.lower():
                autosync = False
        except Exception as e:
            log.error('autosync check failed %r' % e)
            return res

        ' if autosync is not enabled: do nothing '
        if False == autosync:
            return res

        info = self.getTokenInfo()
        syncWindow = self.getSyncWindow()

        # check if the otpval is valid in the sync scope
        res = hmac2Otp.checkOtp(anOtpVal, syncWindow, symetric=True)
        log.debug("found otpval %r in syncwindow (%r): %r" %
                  (anOtpVal, syncWindow, res))

        if res != -1:
            # if former is defined
            if ("otp1c" in info):
                # check if this is consecutive
                otp1c = info.get("otp1c")
                otp2c = res
                log.debug("otp1c: %r, otp2c: %r" % (otp1c, otp2c))
                diff = math.fabs(otp2c - otp1c)
                if (diff > self.resyncDiffLimit):
                    res = -1
                else:
                    T0 = time.time()
                    timeStepping = int(self.getFromTokenInfo("timeStep"))
                    counter = int((T0 / timeStepping) + 0.5)

                    shift = otp2c - counter
                    info["timeShift"] = shift
                    self.setTokenInfo(info)

                # now clean the resync data
                del info["otp1c"]
                self.setTokenInfo(info)

            else:
                log.debug("etting otp1c: %s" % res)
                info["otp1c"] = res
                self.setTokenInfo(info)
                res = -1

        if res == -1:
            msg = "call was not successful"
        else:
            msg = "call was successful"
        log.debug("end. %r: res %r" % (msg, res))

        return res

    @log_with(log)
    def resync(self, otp1, otp2, options=None):
        '''
        resync the token based on two otp values
        - external method to do the resync of the token

        :param otp1: the first otp value
        :type otp1: string

        :param otp2: the second otp value
        :type otp2: string

        :param options: optional token specific parameters
        :type options:  dict or None

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        ret = False

        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError:
            return ret

        secretHOtp = self.token.getHOtpKey()

        self.hashlibStr = self.getFromTokenInfo("hashlib", 'sha1')
        timeStepping = int(self.getFromTokenInfo("timeStep", 30))
        shift = int(self.getFromTokenInfo("timeShift", 0))

        try:
            window = int(self.token.privacyIDEASyncWindow) * timeStepping
        except:
            window = 10 * timeStepping

        log.debug("timestep: %r, syncWindow: %r, timeShift: %r"
                  % (timeStepping, window, shift))

        T0 = time.time() + shift

        log.debug("T0 : %i" % T0)
        # T = (Current Unix time - T0) / timeStepping
        counter = int((T0 / timeStepping) + 0.5)
        log.debug("counter (current time): %i" % counter)

        oCount = self.getOtpCount()

        log.debug("tokenCounter: %r" % oCount)
        log.debug("now checking window %s, timeStepping %s" %
                  (window, timeStepping))
        # check 2nd value
        hmac2Otp = HmacOtp(secretHOtp,
                           counter,
                           otplen,
                           self.getHashlib(self.hashlibStr))
        log.debug("%s in otpkey: %s " % (otp2, secretHOtp))
        res2 = hmac2Otp.checkOtp(otp2,
                                 int(window / timeStepping),
                                 symetric=True)  # TEST -remove the 10
        log.debug("res 2: %r" % res2)
        # check 1st value
        hmac2Otp = HmacOtp(secretHOtp,
                           counter - 1,
                           otplen,
                           self.getHashlib(self.hashlibStr))
        log.debug("%s in otpkey: %s " % (otp1, secretHOtp))
        res1 = hmac2Otp.checkOtp(otp1,
                                 int(window / timeStepping),
                                 symetric=True)  # TEST -remove the 10
        log.debug("res 1: %r" % res1)

        if res1 < oCount:
            # A previous OTP value was used again!
            log.warning("a previous OTP value was used again! tokencounter: "
                        "%i, presented counter %i" %
                        (oCount, res1))
            res1 = -1

        if res1 != -1 and res1 + 1 == res2:
            # here we calculate the new drift/shift between the server time
            # and the tokentime
            tokentime = (res2 + 0.5) * timeStepping
            currenttime = T0 - shift
            new_shift = (tokentime - currenttime)
            log.debug("the counters %r and %r matched. New shift: %r"
                      % (res1, res2, new_shift))
            self.addToTokenInfo('timeShift', new_shift)

            # The OTP value that was used for resync must not be used again!
            self.setOtpCount(res2 + 1)

            ret = True

        if ret is True:
            msg = "resync was successful"
        else:
            msg = "resync was not successful"

        log.debug("end. %s: ret: %r" % (msg, ret))
        return ret

    def getSyncTimeOut(self):
        '''
        get the token sync timeout value

        :return: timeout value in seconds
        :rtype:  int
        '''

        timeOut = int(getFromConfig("AutoResyncTimeout", 5 * 60))
        return timeOut

    @log_with(log)
    def getOtp(self,
               curTime=None,
               do_truncation=True,
               tCounter=None,
               challenge=None):
        '''
        get the next OTP value

        :return: next otp value
        :rtype: string
        '''
        otplen = int(self.token.privacyIDEAOtpLen)
        secretHOtp = self.token.getHOtpKey()
        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1")
        timeStepping = int(self.getFromTokenInfo("timeStep", 30))
        shift = int(self.getFromTokenInfo("timeShift", 0))

        hmac2Otp = HmacOtp(secretHOtp,
                           self.getOtpCount(),
                           otplen,
                           self.getHashlib(self.hashlibStr))

        if tCounter is None:
            tCounter = self.time2float(datetime.datetime.now())
        if curTime:
            tCounter = self.time2float(curTime)

        # we don't need to round here as we have alread float
        counter = int(((tCounter - shift) / timeStepping))
        otpval = hmac2Otp.generate(counter=counter,
                                   inc_counter=False,
                                   do_truncation=do_truncation,
                                   challenge=challenge)

        pin = self.token.getPin()
        combined = "%s%s" % (otpval, pin)
        if getFromConfig("PrependPin") == "True":
            combined = "%s%s" % (pin, otpval)
            
        return (1, pin, otpval, combined)

    @log_with(log)
    def get_multi_otp(self, count=0, epoch_start=0, epoch_end=0,
                      curTime=None, timestamp=None):
        '''
        return a dictionary of multiple future OTP values
        of the HOTP/HMAC token

        :param count: how many otp values should be returned
        :type count: int

        :return: tuple of status: boolean, error: text and the OTP dictionary

        '''
        otp_dict = {"type": "TOTP", "otp": {}}
        ret = False
        error = "No count specified"
        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError:
            return ret

        secretHOtp = self.token.getHOtpKey()

        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1")
        timeStepping = int(self.getFromTokenInfo("timeStep", 30))
        shift = int(self.getFromTokenInfo("timeShift", 0))

        hmac2Otp = HmacOtp(secretHOtp, self.getOtpCount(),
                           otplen, self.getHashlib(self.hashlibStr))

        tCounter = self.time2float(datetime.datetime.now())
        if curTime:
            tCounter = self.time2float(curTime)
        if timestamp:
            tCounter = int(timestamp)

        # we don't need to round here as we have alread float
        counter = int(((tCounter - shift) / timeStepping))

        otp_dict["shift"] = shift
        otp_dict["timeStepping"] = timeStepping

        if count > 0:
            for i in range(0, count):
                otpval = hmac2Otp.generate(counter=counter + i,
                                           inc_counter=False)
                timeCounter = ((counter + i) * timeStepping) + shift
                
                val_time = datetime.datetime.\
                    fromtimestamp(timeCounter).strftime("%Y-%m-%d %H:%M:%S")
                otp_dict["otp"][counter + i] = {'otpval': otpval,
                                                'time': val_time}
            ret = True
            
        return (ret, error, otp_dict)



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216496.431163
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/templates/manage/tokenview.mako'
_template_uri = '/manage/tokenview.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u"\n <button class='ui-button' id='button_losttoken'>")
        # SOURCE LINE 3
        __M_writer(escape(_("Lost token")))
        __M_writer(u"</button>\n <button class='ui-button' id='button_tokeninfo'>")
        # SOURCE LINE 4
        __M_writer(escape(_("Token info")))
        __M_writer(u"</button>\n <button class='ui-button' id='button_resync'>")
        # SOURCE LINE 5
        __M_writer(escape(_("Resync Token")))
        __M_writer(u"</button>\n <button class='ui-button' id='button_tokenrealm'>")
        # SOURCE LINE 6
        __M_writer(escape(_("set token realm")))
        __M_writer(u"</button>\n <button class='ui-button' id='button_getmulti'>")
        # SOURCE LINE 7
        __M_writer(escape(_("get OTP")))
        __M_writer(u"</button>\n \n <a href='")
        # SOURCE LINE 9
        __M_writer(escape(c.help_url))
        __M_writer(u'/tokenview/index.html\' target="_blank">\n\t<img alt="(?)" width=24\n\tsrc="/images/help32.png"  \n\ttitle=\'')
        # SOURCE LINE 12
        __M_writer(escape(_("help on tokenview")))
        __M_writer(u'\'>\n</a>\n \n<table id="token_table" class="flexme2" style="display:none"></table>\n   \n<script type="text/javascript"> \nview_token();\ntokenbuttons();\n</script>\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  functional tests
                
  Dependencies: -

'''
import logging

from privacyidea.tests import TestController, url

log = logging.getLogger(__name__)


class TestSelfserviceController(TestController):

    def createPolicy(self, policy):
        response = self.app.get(url(controller='system', action='setPolicy'),
                                params={'name': 'self01',
                                        'scope': 'selfservice',
                                        'realm': 'myDefRealm',
                                        'action': policy,
                                        'selftest_admin': 'superadmin'
                                        })
        print response
        assert '"status": true' in response
        assert '"setPolicy self01": {' in response
        
    def delete_policy(self):
        response = self.app.get(url(controller='system', action='delPolicy'),
                                params={'name': 'self01',
                                        'selftest_admin': 'superadmin'
                                        })
        print response
        assert '"status": true' in response

    def deleteToken(self, serial):
        response = self.app.get(url(controller='admin', action='remove'),
                                params={'serial': serial,
                                        'selftest_admin': 'superadmin'})

        log.debug(response)

    def test_history(self):
        '''
        Selfservice: Testing history
        '''
        self.createPolicy("history")

        response = self.app.get(url(controller='selfservice',
                                    action='userhistory'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert '"rows": [' in response

        response = self.app.get(url(controller='selfservice',
                                    action='history'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert 'view_audit_selfservice' in response
        
        # we are not allowed to view the history
        self.delete_policy()
        response = self.app.get(url(controller='selfservice',
                                    action='userhistory'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert '"status": false' in response
        assert 'ERR410: The policy settings' in response
        
    def test_reset(self):
        '''
        Selfservice: Testing user reset
        '''
        response = self.app.get(url(controller='selfservice',
                                    action='userreset'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert '"status": false' in response
        assert '"code": -311' in response

        self.createPolicy("reset")
        response = self.app.get(url(controller='selfservice',
                                    action='userreset'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert 'Missing parameter: ' in response
        assert '"code": 905' in response

        response = self.app.get(url(controller='admin',
                                    action='init'),
                                params={'serial': 'reset01',
                                        'type': 'spass',
                                        'user': 'root',
                                        'pin': "secret"
                                        })
        print response
        assert '"status": true' in response

        for _i in "12345678901234567890":
            response = self.app.get(url(controller='validate',
                                        action='check'),
                                    params={'user': 'root',
                                            'pass': 'wrongpass'})
            print response
            assert '"value": false' in response

        response = self.app.get(url(controller='selfservice',
                                    action='userreset'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'reset01'})
        print response
        assert '"status": true' in response
        assert '"reset Failcounter": 1' in response

        response = self.app.get(url(controller='validate',
                                    action='check'),
                                params={'user': 'root',
                                        'pass': 'secret'})
        print response
        assert '"value": true' in response

        response = self.app.get(url(controller='selfservice',
                                    action='reset'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='resetform'>" in response

    def test_resync(self):
        '''
        Selfservice: Testing user resync
        '''
        response = self.app.get(url(controller='selfservice',
                                    action='userresync'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert '"status": false' in response
        assert '"code": -311' in response

        self.createPolicy("resync")
        response = self.app.get(url(controller='selfservice',
                                    action='userresync'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert 'Missing parameter' in response
        assert '"code": 905' in response

        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'user': 'root',
                                        'pin': "secret",
                                        'otpkey': '6161e082d736d3d9d67b'
                                        'c1d4711ff1a81af26160'
                                        })
        print response
        assert '"status": true' in response

        response = self.app.get(url(controller='selfservice',
                                    action='userresync'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'XXXX',
                                        "otp1": "359864",
                                        "otp2": "348448"})
        print response
        assert '"status": false' in response
        assert 'no token found!' in response

        response = self.app.get(url(controller='selfservice',
                                    action='userresync'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'token01',
                                        "otp1": "885497",
                                        "otp2": "696793"})
        print response
        assert '"status": true' in response
        assert '"resync Token": true' in response

        response = self.app.get(url(controller='selfservice', action='resync'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='resyncform'>" in response

    def test_no_selttest_user(self):
        response = self.app.get(url(controller='selfservice',
                                    action='userresync'),
                                params={'serial': 'XXXX',
                                        "otp1": "359864",
                                        "otp2": "348448"})
        print response
        assert '"status": false' in response
        assert "ERR807: When running in selftest" in response

    def test_setmpin(self):
        '''
        Selfservice: setting mOTP PIN
        '''
        response = self.app.get(url(controller='selfservice',
                                    action='usersetmpin'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'XXXX',
                                        'pin': '1234'})
        print response
        assert '"status": false' in response
        assert('"message": "ERR410: The policy settings do not '
               'allow you to issue this request!"' in response)

        self.createPolicy("setMOTPPIN")
        response = self.app.get(url(controller='selfservice',
                                    action='usersetmpin'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert 'Missing parameter: \'pin\'' in response
        assert '"code": 905' in response

        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'user': 'root',
                                        'pin': "secret",
                                        'otpkey': '6161e082d736'
                                        'd3d9d67bc1d4711ff1a81af26160'
                                        })
        print response
        assert '"status": true' in response

        response = self.app.get(url(controller='selfservice',
                                    action='usersetmpin'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'token01',
                                        'pin': '1234'})
        print response
        assert '"status": true' in response
        assert '"set userpin": 1' in response

        response = self.app.get(url(controller='selfservice',
                                    action='setmpin'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='passwordform'>" in response

    def test_setpin(self):
        '''
        Selfservice: testing setting PIN
        '''
        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'spass01',
                                        'type': 'spass',
                                        'user': 'root',
                                        })
        print response
        assert '"status": true' in response

        response = self.app.get(url(controller='selfservice',
                                    action='usersetpin'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'spass01',
                                        'pin': '1234'})
        print response
        assert '"status": false' in response
        assert '"message": "ERR410: The policy settings' in response

        self.createPolicy("setOTPPIN")
        response = self.app.get(url(controller='selfservice',
                                    action='usersetpin'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert 'Missing parameter: \'userpin\'' in response
        assert '"code": 905' in response

        response = self.app.get(url(controller='selfservice',
                                    action='usersetpin'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'spass01',
                                        'userpin': 'secretPin'})
        print response
        assert '"status": true' in response
        assert '"set userpin": 1' in response

        response = self.app.get(url(controller='validate',
                                    action='check'),
                                params={'user': 'root@myDefRealm',
                                        'pass': 'secretPin'})
        print response
        assert '"status": true' in response
        assert '"value": true' in response

        response = self.app.get(url(controller='selfservice', action='setpin'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='passwordform'>" in response

        # testing the index and the list of the tokens
        response = self.app.get(url(controller='selfservice', action='index'),
                                params={'selftest_user': 'root@myDefRealm'})

        print "Selfservice: %r" % response.body
        assert 'Selfservice' in response
        
    def test_get_serial_by_otp(self):
        '''
        selfservice: get serial by otp value
        '''
        self.deleteToken('token01')

        response = self.app.get(url(controller='selfservice',
                                    action='usergetSerialByOtp'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'hmac',
                                        'otp': '885497'})
        print response
        assert '"status": false' in response
        assert('"message": "ERR410: The policy settings do not allow'
               ' you to request a serial by OTP!",' in response)

        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'otpkey': 'c4a3923c8d97e03a'
                                        'f6a12fa40264c54b8429cf0d'
                                        })
        print response
        assert '"status": true' in response

        self.createPolicy("getserial")
        response = self.app.get(url(controller='selfservice',
                                    action='usergetSerialByOtp'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'hmac',
                                        'otp': '459812'})
        print response
        # The token is not found, as it is not in the realm of the user
        assert '"serial": ""' in response

        response = self.app.get(url(controller='admin', action='tokenrealm'),
                                params={'serial': 'token01',
                                        'realms': 'myDefRealm'})
        print response
        assert '"value": 1' in response

        # NOw the token is found
        response = self.app.get(url(controller='selfservice',
                                    action='usergetSerialByOtp'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'hmac',
                                        'otp': '459812'})
        print response
        assert '"serial": "token01"' in response

    def test_assign(self):
        '''
        selfservice: testing assign token and unassign token
        '''
        self.deleteToken('token01')

        # init token
        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'otpkey': 'c4a3923c8d97e03'
                                        'af6a12fa40264c54b8429cf0d'
                                        })
        print response
        assert '"status": true' in response

        # put into realm
        response = self.app.get(url(controller='admin', action='tokenrealm'),
                                params={'serial': 'token01',
                                        'realms': 'myDefRealm'})
        print response
        assert '"value": 1' in response

        # Now try to assign
        response = self.app.get(url(controller='selfservice',
                                    action='userassign'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'token01'})
        print response
        assert '"message": "ERR410: ' in response

        self.createPolicy("assign")
        response = self.app.get(url(controller='selfservice',
                                    action='userassign'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'token01'})
        print response
        assert '"assign token": true' in response
        
        # try to assign to another user
        response = self.app.get(url(controller='selfservice',
                                    action='userassign'),
                                params={'selftest_user': 'horst@myDefRealm',
                                        'serial': 'token01'})
        print response
        assert 'ERR807: The token is already assigned to another' in response
        
        # unassign
        response = self.app.get(url(controller='selfservice',
                                    action='userunassign'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'token01'})
        print response
        assert '"message": "ERR410: The policy settings' in response

        self.createPolicy("unassign")
        response = self.app.get(url(controller='selfservice',
                                    action='userunassign'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'serial': 'token01'})
        print response
        assert '"unassign token": true' in response

        # UI
        response = self.app.get(url(controller='selfservice',
                                    action='assign'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='assignform'>" in response

        response = self.app.get(url(controller='selfservice',
                                    action='unassign'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='unassignform'>" in response

    def test_loadform(self):
        response = self.app.get(url(controller='selfservice',
                                    action='load_form'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'hmac.selfservice.enroll'})
        print response
        assert 'function self_hmac_get_param()' in response
        assert 'function self_hmac_submit()' in response
        
        # wrong scope
        response = self.app.get(url(controller='selfservice',
                                    action='load_form'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'hmac.manage.enroll'})
        print response
        assert 'function self_hmac_get_param()' not in response
        assert 'function self_hmac_submit()' not in response
        
        # scope not splittable
        response = self.app.get(url(controller='selfservice',
                                    action='load_form'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'selfservice.enroll'})
        print response
        assert 'function self_hmac_get_param()' not in response
        assert 'function self_hmac_submit()' not in response
        
    def test_token_call(self):
        response = self.app.get(url(controller='selfservice',
                                    action='token_call'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert 'ERR905: Missing parameter:' in response
        
        response = self.app.get(url(controller='selfservice',
                                    action='token_call'),
                                params={'selftest_user': 'root@myDefRealm',
                                        'type': 'hmac'})
        print response
        assert 'not authorized to call token_call' in response
        
    def test_delete(self):
        '''
        selfservice: testing deleting token
        '''
        self.deleteToken('token01')

        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'otpkey': 'c4a3923c8d97e0'
                                        '3af6a12fa40264c54b8429cf0d',
                                        'user': 'root'
                                        })
        print response
        assert '"status": true' in response

        response = self.app.get(url(controller='selfservice',
                                    action='userdelete'),
                                params={'serial': 'token01',
                                        'selftest_user': 'root@myDefRealm'})
        print response
        assert('"message": "ERR410: The policy settings do not '
               'allow you to issue this request!"' in response)

        self.createPolicy("delete")
        response = self.app.get(url(controller='selfservice',
                                    action='userdelete'),
                                params={'serial': 'token01',
                                        'selftest_user': 'root@myDefRealm'})
        print response
        assert '"delete token": 1' in response

        # UI
        response = self.app.get(url(controller='selfservice', action='delete'))
        print response
        assert "<div id='deleteform'>" in response

    def test_disable(self):
        '''
        selfservice: testing disable and enable token
        '''
        self.deleteToken('token01')

        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'otpkey': 'c4a3923c8d97e03af6a1'
                                        '2fa40264c54b8429cf0d',
                                        'user': 'root'
                                        })
        print response
        assert '"status": true' in response

        # disable
        response = self.app.get(url(controller='selfservice',
                                    action='userdisable'),
                                params={'serial': 'token01',
                                        'selftest_user': 'root@myDefRealm'})
        print response
        assert('"message": "ERR410: The policy settings do not '
               'allow you to issue this request!",' in response)

        self.createPolicy("disable")
        response = self.app.get(url(controller='selfservice',
                                    action='userdisable'),
                                params={'serial': 'token01',
                                        'selftest_user': 'root@myDefRealm'})
        print response
        assert '"disable token": 1' in response

        response = self.app.get(url(controller='admin', action='show'),
                                params={'serial': 'token01'})
        print response
        assert '"privacyIDEA.TokenSerialnumber": "token01",' in response
        assert '"privacyIDEA.Isactive": false' in response

        # now enable again

        response = self.app.get(url(controller='selfservice',
                                    action='userenable'),
                                params={'serial': 'token01',
                                        'selftest_user': 'root@myDefRealm'})
        print response
        assert('"message": "ERR410: The policy settings do not '
               'allow you to issue this request!"' in response)

        self.createPolicy("enable")
        response = self.app.get(url(controller='selfservice',
                                    action='userenable'),
                                params={'serial': 'token01',
                                        'selftest_user': 'root@myDefRealm'})
        print response
        assert '"enable token": 1' in response

        response = self.app.get(url(controller='admin', action='show'),
                                params={'serial': 'token01'})
        print response
        assert '"privacyIDEA.TokenSerialnumber": "token01",' in response
        assert '"privacyIDEA.Isactive": true' in response

        # UI
        response = self.app.get(url(controller='selfservice',
                                    action='disable'))
        print response
        assert "<div id='disableform'>" in response

        response = self.app.get(url(controller='selfservice', action='enable'))
        print response
        assert "<div id='enableform'>" in response

    def test_init(self):
        '''
        selfservice: testing enrollment of token as normal user
        '''
        self.deleteToken('token01')

        response = self.app.get(url(controller='selfservice',
                                    action='userinit'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'otpkey': 'c4a3923c8d97e03af6a12'
                                        'fa40264c54b8429cf0d',
                                        'selftest_user': 'root@myDefRealm'
                                        })
        print response
        assert('"message": "ERR410: The policy settings do not '
               'allow you to issue this request!"' in response)

        self.createPolicy('enrollHMAC')

        response = self.app.get(url(controller='selfservice',
                                    action='userinit'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'otpkey': 'c4a3923c8d97e03af6a12'
                                        'fa40264c54b8429cf0d',
                                        'selftest_user': 'root@myDefRealm'
                                        })
        print response
        assert '"status": true' in response

        response = self.app.get(url(controller='admin', action='show'),
                                params={'serial': 'token01'})
        print response
        assert '"privacyIDEA.TokenSerialnumber": "token01",' in response
        assert '"privacyIDEA.Isactive": true' in response

    def test_webprovision(self):
        '''
        selfservice: testing user webprovision
        '''
        self.deleteToken('token01')
        response = self.app.get(url(controller='selfservice',
                                    action='userwebprovision'),
                                params={'serial': 'token01',
                                        'type': 'hmac',
                                        'selftest_user': 'root@myDefRealm'
                                        })
        print response
        assert('"message": "valid types are \'oathtoken\' and '
               '\'googleauthenticator\' and \'googleauthenticator_time\'. '
               'You provided hmac",' in response)

        response = self.app.get(url(controller='selfservice',
                                    action='userwebprovision'),
                                params={'serial': 'token01',
                                        'type': 'googleauthenticator',
                                        'selftest_user': 'root@myDefRealm'
                                        })
        print response
        assert('"message": "ERR410: The policy settings do not allow '
               'you to issue this request!"' in response)

        self.createPolicy('webprovisionGOOGLE')

        response = self.app.get(url(controller='selfservice',
                                    action='userwebprovision'),
                                params={'prefix': 'LSGO',
                                        'type': 'googleauthenticator',
                                        'selftest_user': 'root@myDefRealm'
                                        })
        print response
        assert '"url": "otpauth://hotp/LSGO' in response

        # test
        response = self.app.get(url(controller='admin', action='show'),
                                params={'user': 'root'})
        print response
        assert '"privacyIDEA.TokenSerialnumber": "LSGO' in response
        assert '"privacyIDEA.Isactive": true' in response

        # UI

        response = self.app.get(url(controller='selfservice',
                                    action='webprovisiongoogletoken'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='googletokenform'>" in response

    def test_webprovision_motp(self):
        '''
        selfservice: testing user webprovision for MOTP token
        '''
        self.deleteToken('token01')
        response = self.app.get(url(controller='selfservice',
                                    action='userinit'),
                                params={'type': 'motp',
                                        'genkey': "1",
                                        'selftest_user': 'root@myDefRealm',
                                        "otppin": "1234"
                                        })
        print response
        assert('ERR410: The policy settings do not allow you to'
               ' issue this request!' in response)

        self.createPolicy('motp_webprovision')
        response = self.app.get(url(controller='selfservice',
                                    action='userinit'),
                                params={'type': 'motp',
                                        'genkey': "1",
                                        'selftest_user': 'root@myDefRealm',
                                        'otppin': "1234"
                                        })
        print response
        assert '"value": true' in response
        assert '"motpurl": {' in response

    def test_ocra(self):
        '''
        TODO selfservice: testing ocra
        '''
        pass

    def test_getmultiotp(self):
        '''
        TODO selfservice: testing getting multiple otps
        '''
        pass
    
    def test_forms(self):
        '''
        Selfservice: Testing different forms
        '''
        response = self.app.get(url(controller='selfservice', action='getotp'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='getotpform'>" in response
        
        response = self.app.get(url(controller='selfservice',
                                    action='webprovisionoathtoken'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='oathtokenform'>" in response
        
        response = self.app.get(url(controller='selfservice',
                                    action='activateqrtoken'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<div id='activateqrform'>" in response
        
        response = self.app.get(url(controller='selfservice',
                                    action='custom_style'),
                                params={'selftest_user': 'root@myDefRealm'})
        print "Custom style:", response
        assert response.headers.get("Content-Type") == "text/css"

    def test_user_tokenlist(self):
        response = self.app.get(url(controller='selfservice',
                                    action='usertokenlist'),
                                params={'selftest_user': 'root@myDefRealm'})
        print response
        assert "<ul>" in response



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216580.949147
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/templates/manage/policies.mako'
_template_uri = '/manage/policies.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'    \n\n<a id=policy_export>')
        # SOURCE LINE 4
        __M_writer(escape(_("Export policies")))
        __M_writer(u'</a>\n\n<button id=policy_import>')
        # SOURCE LINE 6
        __M_writer(escape(_("Import policies")))
        __M_writer(u"</button>\n\n<a href='")
        # SOURCE LINE 8
        __M_writer(escape(c.help_url))
        __M_writer(u'/policies/index.html\' target="_blank">\n\t<img alt="(?)" width=24\n\tsrc="/images/help32.png"  \n\ttitle=\'')
        # SOURCE LINE 11
        __M_writer(escape(_("Open help on policies")))
        __M_writer(u'\'>\n</a>\n\n<table id="policy_table" class="flexme2" style="display:none"></table>\n   \n   \n<table>\n  <tr>\n  \t<td><label for=policy_active>')
        # SOURCE LINE 19
        __M_writer(escape(_("Active")))
        __M_writer(u'</label></td>\n  \t<td><input type="checkbox" name="policy_active" id="policy_active" value="True"></td>\n  </tr>\t\n  <tr>\n  \t<td><label for=policy_name>')
        # SOURCE LINE 23
        __M_writer(escape(_("Policy name")))
        __M_writer(u'</label></td>\n  \t<td><input type="text" class="required"  id="policy_name" size="40" maxlength="80" \n  \t\ttitle=\'')
        # SOURCE LINE 25
        __M_writer(escape(_("The name of the policy")))
        __M_writer(u"'></td>\n  </tr>\n  <tr>\n\t<td><label for=policy_scope_combo>")
        # SOURCE LINE 28
        __M_writer(escape(_("Scope")))
        __M_writer(u'</label></td>\n\t<td>\n\t<select id=\'policy_scope_combo\'>\n\t<option value="_">')
        # SOURCE LINE 31
        __M_writer(escape(_("__undefined__")))
        __M_writer(u'</option>\n')
        # SOURCE LINE 32
        for scope in c.polDefs.keys():
            # SOURCE LINE 33
            __M_writer(u'\t<option value="')
            __M_writer(escape(scope))
            __M_writer(u'">')
            __M_writer(escape(scope))
            __M_writer(u'</option>\n')
        # SOURCE LINE 35
        __M_writer(u'\t</select>\n\t</td>\n  </tr>\n    <tr>\n  \t<td><label for="policy_action">')
        # SOURCE LINE 39
        __M_writer(escape(_("Action")))
        __M_writer(u'</label></td>\n  \t<td><input type="text" class="required"  id="policy_action" size="40" maxlength="200" \n  \t\ttitle=\'')
        # SOURCE LINE 41
        __M_writer(escape(_("The action that should be allowed. These are actions like: enrollSMS, enrollMOTP...The actions may be comma separated.")))
        __M_writer(u'\'>\n  \t</td><td>\n  \t\t<span id="action_info" class="help_text"> </span>\n  \t\t</td>\n  </tr>\n  <tr>\n  \t<td><label for="policy_user">')
        # SOURCE LINE 47
        __M_writer(escape(_("User")))
        __M_writer(u'</label></td>\n  \t<td><input type="text"  id="policy_user" size="40" maxlength="80" \n  \t\ttitle=\'')
        # SOURCE LINE 49
        __M_writer(escape(_("The user or usergroup the policy should apply to")))
        __M_writer(u'\'></td>\n  </tr>\n    <tr>\n  \t<td><label for="policy_realm">')
        # SOURCE LINE 52
        __M_writer(escape(_("Realm")))
        __M_writer(u'</label></td>\n  \t<td><input type="text" class="required"  id="policy_realm" size="40" maxlength="80" \n  \t\ttitle=\'')
        # SOURCE LINE 54
        __M_writer(escape(_("The realm the policy applies to")))
        __M_writer(u'\'></td>\n  </tr>\n   <tr>\n  \t<td><label for="policy_client">')
        # SOURCE LINE 57
        __M_writer(escape(_("Client")))
        __M_writer(u'</label></td>\n  \t<td><input type="text"  id="policy_client" size="40" maxlength="120" \n  \t\ttitle=\'')
        # SOURCE LINE 59
        __M_writer(escape(_("Comma separated list of client IPs and Subnets.")))
        __M_writer(u"'></td>\n  </tr>\n  <tr>\n  \t<td><label for=policy_time>")
        # SOURCE LINE 62
        __M_writer(escape(_("Time")))
        __M_writer(u'</label></td>\n  \t<td><input type="text"  id="policy_time" size="40" maxlength="80" \n  \t\ttitle=\'')
        # SOURCE LINE 64
        __M_writer(escape(_("The time on which the policy should be applied")))
        __M_writer(u"'></td>\n  </tr>\n  <tr>\n  \t<td></td><td>\n  \t<button  id=button_policy_add>")
        # SOURCE LINE 68
        __M_writer(escape(_("set policy")))
        __M_writer(u'</button>\n  \t<button  id=button_policy_delete>')
        # SOURCE LINE 69
        __M_writer(escape(_("delete policy")))
        __M_writer(u'</button>\n  \t</td>\n  </tr>\n</table> \n<script type="text/javascript"> \n\tview_policy();\n</script>\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''

import logging
from privacyidea.tests import TestController, url

log = logging.getLogger(__name__)


class TestGetSerialController(TestController):

    def setUp(self):
        self.initToken()

    def createHOtpToken(self, hashlib, serial):
        '''
        // Seed for HMAC-SHA1 - 20 bytes
        String seed = "3132333435363738393031323334353637383930";
        // Seed for HMAC-SHA256 - 32 bytes
        String seed32 = "3132333435363738393031323334353637383930" +
        "313233343536373839303132";
        // Seed for HMAC-SHA512 - 64 bytes
        String seed64 = "3132333435363738393031323334353637383930" +
        "3132333435363738393031323334353637383930" +
        "3132333435363738393031323334353637383930" +
        "31323334";
        '''
        ##

        if (hashlib == "SHA512"):
            otpkey = "31323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334"
        elif (hashlib == "SHA256"):
            otpkey = "3132333435363738393031323334353637383930313233343536373839303132"
        else:
            otpkey = "3132333435363738393031323334353637383930"
        parameters = {
                          "serial"  : serial,
                          "type"    : "HMAC",
                          # 64 byte key
                          "otpkey"  : otpkey,
                          "otppin"  : "1234",
                          "pin"     : "pin",
                          "otplen"  : 6,
                          "description" : "time based HMAC TestToken1",
                          "hashlib": hashlib,
                          }


        response = self.app.get(url(controller='admin', action='init'), params=parameters)
        assert '"value": true' in response


    def removeTokenBySerial(self, serial):

        parameters = {
                      "serial": serial,
                      }

        response = self.app.get(url(controller='admin', action='remove'), params=parameters)
        return response



    def setTokenRealm(self, serial, realms):
        parameters = { "serial" : serial,
                       "realms" : realms}

        response = self.app.get(url(controller="admin", action="tokenrealm"), params=parameters)
        return response


    def initToken(self):
        '''
        init two tokens in two realms
        '''

        self.createHOtpToken("SHA1", "oath_mydef")
        '''
        Your OTP with number 2 is 359152.
        Your OTP with number 3 is 969429.
        Your OTP with number 4 is 338314.
        Your OTP with number 5 is 254676.
        Your OTP with number 6 is 287922.
        '''
        self.createHOtpToken("SHA256", "oath_myrealm")
        '''
        Your OTP with number 2 is 072768.
        Your OTP with number 3 is 797306.
        Your OTP with number 4 is 038285.
        Your OTP with number 5 is 143665.
        '''

        # create resolvers - this is a legacy interface
        # but as this is still used in the web gui, we leave this here
        parameters = { "passwdresolver.fileName.mdef" : "%(here)s/tests/testdata/def-passwd",
                       "passwdresolver.fileName.mrealm" :"%(here)s/tests/testdata/def-passwd" }
        resp = self.app.get(url(controller="system", action="setConfig"), parameters)
        print resp
        assert '"status": true' in resp

        # create realms
        parameters = { "realm" : "mydef",
                       "resolvers" : "privacyidea.lib.resolvers.PasswdIdResolver.IdResolver.mdef" }
        resp = self.app.get(url(controller="system", action="setRealm"), parameters)
        print resp
        assert '"status": true' in resp

        ## legacy syntax for resolver reference
        parameters = { "realm" : "myrealm",
                       "resolvers" : "privacyidea.lib.resolvers.passwdresolver.mrealm" }
        resp = self.app.get(url(controller="system", action="setRealm"), parameters)
        print resp
        assert '"status": true' in resp

        resp = self.setTokenRealm("oath_mydef", "mydef")
        print resp
        assert '"status": true' in resp
        resp = self.setTokenRealm("oath_myrealm", "myrealm")
        print resp
        assert '"status": true' in resp


    #def test_02_check_realms(self):
    #
    #    parameters = {}
    #    response = self.app.get(url(controller='admin', action='show'),params=parameters)
    #    print response
    #    #assert '"privacyIDEA.RealmNames": [\n"myrealm"\n]' in resp

    def test_02_token01_success(self):
        '''
        test for the otp of the first token, with all realms
        '''

        parameters = {'otp' : '359152'}
        response = self.app.get(url(controller='admin', action='getSerialByOtp'), params=parameters)
        print response
        assert '"serial": "oath_mydef"' in response



        '''
        test for the otp of the first token, with only in realm mydef
        But it fails, due to same OTP value!
        '''

        parameters = {'otp' : '359152',
                      'realm': 'mydef'}
        response = self.app.get(url(controller='admin', action='getSerialByOtp'), params=parameters)
        print response
        assert '"serial": ""' in response

        '''
        test for the otp of the first token, with only in realm mydef
        '''
        parameters = {'otp' : '969429',
                      'realm': 'mydef'}
        response = self.app.get(url(controller='admin', action='getSerialByOtp'), params=parameters)
        print response
        assert '"serial": "oath_mydef"' in response


        '''
        The OTP of the first token shall not be found in the second realm
        '''

        parameters = {'otp' : '338314',
                      'realm': 'myrealm'}
        response = self.app.get(url(controller='admin', action='getSerialByOtp'), params=parameters)
        print response
        assert '"serial": ""' in response




# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.944568
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/templates/manage/start.mako'
_template_uri = '/manage/start.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = ['sidebar']


def _mako_get_namespace(context, name):
    try:
        return context.namespaces[(__name__, name)]
    except KeyError:
        _mako_generate_namespaces(context)
        return context.namespaces[(__name__, name)]
def _mako_generate_namespaces(context):
    pass
def _mako_inherit(template, context):
    _mako_generate_namespaces(context)
    return runtime._inherit_from(context, u'manage-base.mako', _template_uri)
def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n\n\n')
        # SOURCE LINE 43
        __M_writer(u'\n\n\n<div id="main">\n\n<div class=info_box id=info_box>\n<span id=info_text>\n</span>\n<button id=button_info_text>OK</button>\n</div>\n<script>\n\t$(\'#info_box\').hide();\n\t$("#button_info_text").button("enable");\n\t$(\'#button_info_text\').click(function(){\n\t\t$(\'#info_box\').hide(\'blind\',{},500);\n\t});\n</script>\n\n<div id="tabs">\n\t<ul>\n\t\t<li><a href="/manage/tokenview" onclick="view_clicked(\'tokenview\');"><span>')
        # SOURCE LINE 63
        __M_writer(escape(_("Token View")))
        __M_writer(u'</span></a></li>\n\t\t<li><a href="/manage/userview" onclick="view_clicked(\'userview\');"><span>')
        # SOURCE LINE 64
        __M_writer(escape(_("User View")))
        __M_writer(u'</span></a></li>\n\t\t<li><a href="/manage/policies" onclick="view_clicked(\'policies\');"><span>')
        # SOURCE LINE 65
        __M_writer(escape(_("Policies")))
        __M_writer(u'</span></a></li>\n\t\t<li><a href="/manage/machines" onclick="view_clicked(\'machines\');"><span>')
        # SOURCE LINE 66
        __M_writer(escape(_("Machines")))
        __M_writer(u'</span></a></li>\n\t\t<li><a href="/manage/audittrail" onclick="view_clicked(\'audit\');"><span>')
        # SOURCE LINE 67
        __M_writer(escape(_("Audit Trail")))
        __M_writer(u'</span></a></li>\n\t</ul>\n</div>\n\n<!--\n\n<div id="logAccordionResizer" style="padding:10px; width:97%; height:120px;" class="ui-widget-content">\n<div id="logAccordion" class="ui-accordion">\n<h3 class=""ui-accordion-header"><a href="#">Log</a></h3>\n\t<div id="logText">\n\t</div>\n</div>\n<span class="ui-icon ui-icon-grip-dotted-horizontal" style="margin:2px auto;"></span>\n</div>\n-->\n\n<div id=\'errorDiv\'></div>\n<div id=\'successDiv\'></div>\n\n</div>  <!-- end of main-->\n')
        return ''
    finally:
        context.caller_stack._pop_frame()


def render_sidebar(context):
    __M_caller = context.caller_stack._push_frame()
    try:
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 6
        __M_writer(u'\n    <div id="realms">\n    ')
        # SOURCE LINE 8
        __M_writer(escape(_("Realms")))
        __M_writer(u': <select id=realm></select>\n    </div>\n    \n    <fieldset>\n    <legend id="selected_tokens_header">')
        # SOURCE LINE 12
        __M_writer(escape(_("selected tokens")))
        __M_writer(u'</legend>\n    <div id="selected_tokens"></div>\n    <button class=\'action-button\' id=\'button_unselect\'>\n    ')
        # SOURCE LINE 15
        __M_writer(escape(_("clear token selection")))
        __M_writer(u'\n    </button>\n    </fieldset>\n    \n    <span id="selected_users_header">')
        # SOURCE LINE 19
        __M_writer(escape(_("selected users")))
        __M_writer(u'</span>\n    <div id="selected_users"></div>\n    \n    <span id="selected_machine_header">')
        # SOURCE LINE 22
        __M_writer(escape(_("selected machine")))
        __M_writer(u'</span>\n    <div id="selected_machine"></div>\n    \n\t<div class=button_bar_token id=button_bar_token>\n    <button class=\'action-button\' id=\'button_enroll\'>')
        # SOURCE LINE 26
        __M_writer(escape(_("enroll")))
        __M_writer(u"</button>\n   \n    <button class='action-button' id='button_assign'>")
        # SOURCE LINE 28
        __M_writer(escape(_("assign")))
        __M_writer(u"</button>\n\n    <button class='action-button' id='button_unassign'>")
        # SOURCE LINE 30
        __M_writer(escape(_("unassign")))
        __M_writer(u"</button>\n\n    <button class='action-button' id='button_enable'>")
        # SOURCE LINE 32
        __M_writer(escape(_("enable")))
        __M_writer(u"</button>\n\n    <button class='action-button' id='button_disable'>")
        # SOURCE LINE 34
        __M_writer(escape(_("disable")))
        __M_writer(u"</button>\n\n    <button class='action-button' id='button_setpin'>")
        # SOURCE LINE 36
        __M_writer(escape(_("set PIN")))
        __M_writer(u"</button>\n\n    <button class='action-button' id='button_resetcounter'>")
        # SOURCE LINE 38
        __M_writer(escape(_("Reset failcounter")))
        __M_writer(u"</button>\n\n    <button class='action-button' id='button_delete'>")
        # SOURCE LINE 40
        __M_writer(escape(_("delete")))
        __M_writer(u'</button>\n\t</div>\n\t\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''

import logging
from privacyidea.tests import TestController, url
import privacyidea.lib.ImportOTP
#FIXME: missing import for PSKC...
#import privacyideaee.lib.ImportOTP
#
# PSCK, eToken DAT...

log = logging.getLogger(__name__)


XML_PSKC = '''<?xml version="1.0" encoding="UTF-8"?>

<KeyContainer Version="1.0" xmlns ="urn:ietf:params:xml:ns:keyprov:pskc">
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>Feitian Technology Co.,Ltd</Manufacturer>
      <SerialNo>1000133508267</SerialNo>
    </DeviceInfo>
    <Key Id="1000133508267" Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:hotp">
      <AlgorithmParameters>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
      </AlgorithmParameters>
      <Data>
        <Secret>
          <PlainValue>PuMnCivln/14Ii3DNhR4/1zGN5A=</PlainValue>
        </Secret>
        <Counter>
          <PlainValue>0</PlainValue>
        </Counter>
      </Data>
      <Policy>
        <StartDate>2012-08-01T00:00:00Z</StartDate>
        <ExpiryDate>2037-12-31T00:00:00Z</ExpiryDate>
      </Policy>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>Feitian Technology Co.,Ltd</Manufacturer>
      <SerialNo>1000133508255</SerialNo>
    </DeviceInfo>
    <Key Id="1000133508255" Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:hotp">
      <AlgorithmParameters>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
      </AlgorithmParameters>
      <Data>
        <Secret>
          <PlainValue>wRjcslncyKj//L1oaDVQbAvCNnI=</PlainValue>
        </Secret>
        <Counter>
          <PlainValue>0</PlainValue>
        </Counter>
      </Data>
      <Policy>
        <StartDate>2012-08-01T00:00:00Z</StartDate>
        <ExpiryDate>2037-12-31T00:00:00Z</ExpiryDate>
      </Policy>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>Feitian Technology Co.,Ltd</Manufacturer>
      <SerialNo>2600124809778</SerialNo>
    </DeviceInfo>
    <Key Id="2600124809778" Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:totp">
      <AlgorithmParameters>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
      </AlgorithmParameters>
      <Data>
        <Secret>
          <PlainValue>MRffGnGNJKmo8uSW313HCvGNIYM=</PlainValue>
        </Secret>
        <Time>
          <PlainValue>0</PlainValue>
        </Time>
        <TimeInterval>
          <PlainValue>60</PlainValue>
        </TimeInterval>
      </Data>
      <Policy>
        <StartDate>2012-08-01T00:00:00Z</StartDate>
        <ExpiryDate>2037-12-31T00:00:00Z</ExpiryDate>
      </Policy>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>Feitian Technology Co.,Ltd</Manufacturer>
      <SerialNo>2600124809787</SerialNo>
    </DeviceInfo>
    <Key Id="2600124809787" Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:totp">
      <AlgorithmParameters>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
      </AlgorithmParameters>
      <Data>
        <Secret>
          <PlainValue>9O9PX9g20x74kIcaLLrGiwMUReM=</PlainValue>
        </Secret>
        <Time>
          <PlainValue>0</PlainValue>
        </Time>
        <TimeInterval>
          <PlainValue>60</PlainValue>
        </TimeInterval>
      </Data>
      <Policy>
        <StartDate>2012-08-01T00:00:00Z</StartDate>
        <ExpiryDate>2037-12-31T00:00:00Z</ExpiryDate>
      </Policy>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>Feitian Technology Co.,Ltd</Manufacturer>
      <SerialNo>2600135004012</SerialNo>
    </DeviceInfo>
    <Key Id="2600135004012" Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:totp">
      <AlgorithmParameters>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
      </AlgorithmParameters>
      <Data>
        <Secret>
          <PlainValue>A0DxFX1zRVTsxJlMKFsDXuNQYcI=</PlainValue>
        </Secret>
        <Time>
          <PlainValue>0</PlainValue>
        </Time>
        <TimeInterval>
          <PlainValue>60</PlainValue>
        </TimeInterval>
      </Data>
      <Policy>
        <StartDate>2012-08-01T00:00:00Z</StartDate>
        <ExpiryDate>2037-12-31T00:00:00Z</ExpiryDate>
      </Policy>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>Feitian Technology Co.,Ltd</Manufacturer>
      <SerialNo>2600135004013</SerialNo>
    </DeviceInfo>
    <Key Id="2600135004013" Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:totp">
      <AlgorithmParameters>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
      </AlgorithmParameters>
      <Data>
        <Secret>
          <PlainValue>NSLuCF/qeQPsqY7Sod4anJMjIBg=</PlainValue>
        </Secret>
        <Time>
          <PlainValue>0</PlainValue>
        </Time>
        <TimeInterval>
          <PlainValue>60</PlainValue>
        </TimeInterval>
      </Data>
      <Policy>
        <StartDate>2012-08-01T00:00:00Z</StartDate>
        <ExpiryDate>2037-12-31T00:00:00Z</ExpiryDate>
      </Policy>
    </Key>
  </KeyPackage>
</KeyContainer>
'''

class TestImportOTP(TestController):

    def test_parse_DAT(self):
        '''
        Test to parse of eToken dat file format - import
        '''
        #FIXME EE import
        return
        data = '''
# ===== SafeWord Authenticator Records $Version: 100$ =====
dn: sccAuthenticatorId=RAINER01
objectclass: sccCompatibleToken
sccAuthenticatorId: RAINER01
sccTokenType: eToken-PASS-ES
sccTokenData: sccKey=E26BF3661C254BBAB7370296A6DE60D7AC8E0141;sccMode=E;sccPwLen=6;sccVer=6.20;
sccSignature:MC0CFGxPAjrb0zg7MwFzrPibnC70klMnAhUAwZzVdGBaKGjA0djXrGuv6ejTtII=

dn: sccAuthenticatorId=RAINER02
objectclass: sccCompatibleToken
sccAuthenticatorId: RAINER02
sccTokenType: eToken-PASS-TS
sccTokenData: sccKey=535CC2CB9DEA0B55B0A2D585EAB648EBCE73AC8B;sccMode=T;sccPwLen=6;sccVer=6.20;sccTick=30;sccPrTime=2013/03/12 00:00:00
sccSignature: MC4CFQDju23MCRqmkWC7Z9sVDB0y0TeEOwIVAOIibmqMFxhPiY7mLlkt5qmRT/xn        '''

        #from privacyideaee.lib.ImportOTP.eTokenDat import parse_dat_data
        import privacyideaee.lib.ImportOTP.eTokenDat
        TOKENS = privacyideaee.lib.ImportOTP.eTokenDat.parse_dat_data(data, '1.1.2000')
        log.error(TOKENS)
        assert(len(TOKENS) == 2)
        assert(TOKENS.get("RAINER02") != None)
        assert(TOKENS.get("RAINER01") != None)
        return

    def test_import_DAT(self):
        '''
        Test to import of eToken dat file format
        '''
        #FIXME EE import
        return
        data = '''
# ===== SafeWord Authenticator Records $Version: 100$ =====
dn: sccAuthenticatorId=RAINER01
objectclass: sccCompatibleToken
sccAuthenticatorId: RAINER01
sccTokenType: eToken-PASS-ES
sccTokenData: sccKey=E26BF3661C254BBAB7370296A6DE60D7AC8E0141;sccMode=E;sccPwLen=6;sccVer=6.20;
sccSignature:MC0CFGxPAjrb0zg7MwFzrPibnC70klMnAhUAwZzVdGBaKGjA0djXrGuv6ejTtII=

dn: sccAuthenticatorId=RAINER02
objectclass: sccCompatibleToken
sccAuthenticatorId: RAINER02
sccTokenType: eToken-PASS-TS
sccTokenData: sccKey=535CC2CB9DEA0B55B0A2D585EAB648EBCE73AC8B;sccMode=T;sccPwLen=6;sccVer=6.20;sccTick=30;sccPrTime=2013/03/12 00:00:00
sccSignature: MC4CFQDju23MCRqmkWC7Z9sVDB0y0TeEOwIVAOIibmqMFxhPiY7mLlkt5qmRT/xn        '''

        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':data,
                                         'type':'dat',
                                         'startdate':'1.1.2000', })
        print response
        assert '"imported": 2' in response

        data = ""
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':data,
                                         'type':'dat',
                                         'startdate':'1.1.2000', })
        print response
        assert 'Error loading tokens. File or Type empty' in response

        data = """
####
"""
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file': data,
                                         'type': 'dat',
                                         'startdate': '1.1.2000', })
        print response
        assert '"imported": 0' in response

        ## test: no startdate
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':data,
                                         'type':'dat',
                                         })
        print response
        assert '"imported": 0' in response

        ## test: wrong startdate
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':data,
                                         'type':'dat',
                                         'startdate': '2000-12-12', })
        print response
        assert '"imported": 0' in response

    def test_parse_PSKC_OCRA(self):
        '''
        Test import OCRA via PSCK
        '''
        #FIXME EE import
        return
        xml = '''<?xml version="1.0" encoding="UTF-8"?>
<KeyContainer Version="1.0"
              Id="KC20130122"
              xmlns="urn:ietf:params:xml:ns:keyprov:pskc"
              xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
              xmlns:xenc="http://www.w3.org/2001/04/xmlenc#">
     <EncryptionKey>
         <ds:KeyName>Pre-shared-key</ds:KeyName>
     </EncryptionKey>
     <MACMethod Algorithm="http://www.w3.org/2000/09/xmldsig#hmac-sha1">
         <MACKey>
             <xenc:EncryptionMethod
             Algorithm="http://www.w3.org/2001/04/xmlenc#aes128-cbc"/>
             <xenc:CipherData>
                 <xenc:CipherValue>OdudVkgsZywiwE1HqPGOJtHmBl+6HzJkylgDrZU9gcflyCddzO+cxEwzYIlOiwrE</xenc:CipherValue>
             </xenc:CipherData>
         </MACKey>
     </MACMethod>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>NagraID Security</Manufacturer>
      <SerialNo>306EUO4-00960</SerialNo>
      <Model>306E</Model>
      <IssueNo>880479B6A2CA2080</IssueNo>
    </DeviceInfo>
    <Key Id="880479B6A2CA2080"
         Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:ocra">
    <AlgorithmParameters>
        <Suite>OCRA-1:HOTP-SHA1-6:C-QN08-PSHA1</Suite>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
    </AlgorithmParameters>
      <Data>
        <Secret>
          <EncryptedValue>
            <xenc:EncryptionMethod
                  Algorithm="http://www.w3.org/2001/04/xmlenc#aes128-cbc"/>
            <xenc:CipherData>
              <xenc:CipherValue>VHdEP8TXnMmE3yiAnB5Fx+SQ85UXCNAxH7IyOixJpUZHMk9GTdFYWNsxZp8jVpfp</xenc:CipherValue>
            </xenc:CipherData>
          </EncryptedValue>
          <ValueMAC>uQ1Bef+XVXHQoW4ZzyQ/cv/9zYA=</ValueMAC>
        </Secret>
        <Counter>
          <PlainValue>0</PlainValue>
        </Counter>
      </Data>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>NagraID Security</Manufacturer>
      <SerialNo>306EUO4-00954</SerialNo>
      <Model>306E</Model>
      <IssueNo>880489CFA2CA2080</IssueNo>
    </DeviceInfo>
    <Key Id="880489CFA2CA2080"
         Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:ocra">
    <AlgorithmParameters>
       <Suite>OCRA-1:HOTP-SHA1-6:C-QN08-PSHA1</Suite>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
    </AlgorithmParameters>
      <Data>
        <Secret>
          <EncryptedValue>
            <xenc:EncryptionMethod
                  Algorithm="http://www.w3.org/2001/04/xmlenc#aes128-cbc"/>
            <xenc:CipherData>
              <xenc:CipherValue>YTvA1cSntb4cPJHPFkJwuSZkAsLPo+o1EJPA22DeijZRaKhJAwArQKbwDwSmNrR1</xenc:CipherValue>
            </xenc:CipherData>
          </EncryptedValue>
          <ValueMAC>N8QGRQ7yKd8suyUgaEVme7f0HrA=</ValueMAC>
        </Secret>
        <Counter>
          <PlainValue>0</PlainValue>
        </Counter>
      </Data>
    </Key>
  </KeyPackage>
  <KeyPackage>
    <DeviceInfo>
      <Manufacturer>NagraID Security</Manufacturer>
      <SerialNo>306EUO4-00958</SerialNo>
      <Model>306E</Model>
      <IssueNo>880497B3A2CA2080</IssueNo>
    </DeviceInfo>
    <Key Id="880497B3A2CA2080"
         Algorithm="urn:ietf:params:xml:ns:keyprov:pskc:ocra">
    <AlgorithmParameters>
        <Suite>OCRA-1:HOTP-SHA1-6:C-QN08-PSHA1</Suite>
        <ResponseFormat Length="6" Encoding="DECIMAL"/>
    </AlgorithmParameters>
      <Data>
        <Secret>
          <EncryptedValue>
            <xenc:EncryptionMethod
                  Algorithm="http://www.w3.org/2001/04/xmlenc#aes128-cbc"/>
            <xenc:CipherData>
              <xenc:CipherValue>BdxW7Pb46LafGV8k2zDQ48ujoyYX7M+JumfS3Wx5dP1E9y5By/97QTMiGkzJrcWj</xenc:CipherValue>
            </xenc:CipherData>
          </EncryptedValue>
          <ValueMAC>WGhmLhbGn4Dksa7lHKfKOqbsJhU=</ValueMAC>
        </Secret>
        <Counter>
          <PlainValue>0</PlainValue>
        </Counter>
      </Data>
    </Key>
  </KeyPackage>
</KeyContainer>
        '''
        from privacyideaee.lib.ImportOTP.PSKC import parsePSKCdata
        TOKENS = parsePSKCdata(xml,
                 preshared_key_hex="4A057F6AB6FCB57AB5408E46A9835E68",
                 do_checkserial=False)
        log.error(TOKENS)
        assert(len(TOKENS) == 3)
        assert(TOKENS.get("306EUO4-00954") != None)
        assert(TOKENS.get("306EUO4-00958") != None)
        assert(TOKENS.get("306EUO4-00960") != None)


    def test_parse_HOTP_PSKC(self):
        '''
        Test import HOTP via PSKC
        '''
        #FIXME EE import
        return
        TOKENS = privacyideaee.lib.ImportOTP.PSKC.parsePSKCdata(XML_PSKC,
                                                           do_checkserial=False)
        log.error(TOKENS)
        assert(len(TOKENS) == 6)


    def test_parse_Yubikey_CSV(self):
        '''
        Test the parsing of Yubikey CSV file
        '''
        csv= '''
        Static Password: Scan Code,17.04.12 12:25,1,051212172c092728,,,,,0,0,0,0,0,0,0,0,0,0
        Static Password: Scan Code,17.04.12 12:27,1,282828051212172c092728,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,17.04.12 12:29
        Static Password: Scan Code,17.04.12 12:29,1,2828282828051212172c092728,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 16:43
        Yubico OTP,11.12.13 16:43,1,cccccccirblh,b321173a2fb8,6faa3ce885fcd5eda7efa5195e5a5d44,,,0,0,0,0,0,0,0,0,0,1
        Yubico OTP,11.12.13 16:43,1,ccccccbgbhkl,9b19889fc5c1,11261596dbbeae6538b26ce0cfd4f9c9,,,0,0,0,0,0,0,0,0,0,1
        LOGGING START,11.12.13 18:55
        OATH-HOTP,11.12.13 18:55,1,cccccccccccc,,916821d3a138bf855e70069605559a206ba854cd,,,0,0,0,6,0,0,0,0,0,0
        LOGGING START,11.12.13 18:58
        Yubico OTP,11.12.13 18:58,1,,,a54c68c7c3d5a1fec8a0c85b8d60765b,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 19:00
        OATH-HOTP,11.12.13 19:00,1,cccccccccccc,,1390612c06ec6dd0fa077ce99bf9c86d2c058f42,,,0,0,0,6,0,0,0,0,0,0
        LOGGING START,11.12.13 19:01
        OATH-HOTP,11.12.13 19:01,1,,,d41845578effd750887edc70f04df754603e2b63,,,0,0,0,6,0,0,0,0,0,0
        LOGGING START,11.12.13 19:05
        Static Password: Scan Code,11.12.13 19:05,1,040716040416070416041607041607,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 19:06
        Static Password: Scan Code,11.12.13 19:06,1,1e1f201a1407048796,,,,,0,0,0,0,0,0,0,0,0,0
        Static Password: Scan Code,11.12.13 19:06,1,1e1f201a1407048796,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 19:07
        Static Password,11.12.13 19:07,1,,ba23877e747c,fe8abdf8c0c9b6ad6a1daabefa4d50b3,,,0,0,0,0,0,0,0,0,0,0
        Static Password,11.12.13 19:08,1,,d5a3d50327dc,0e8e37b0e38b314a56748c030f58d21d,,,0,0,0,0,0,0,0,0,0,0
        '''
        TOKENS = privacyidea.lib.ImportOTP.parseYubicoCSV(csv)
        print TOKENS
        print len(TOKENS)
        assert len(TOKENS) == 3

    def test_parse_XML(self):
        '''
        Test an SafeNet XML import
        '''
        xml = '''
        <Tokens>
        <Token serial="00040008CFA5">
        <CaseModel>5</CaseModel>
        <Model>101</Model>
        <ProductionDate>02/19/2009</ProductionDate>
        <ProductName>Safeword Alpine</ProductName>
        <Applications>
        <Application ConnectorID="{ab1397d2-ddb6-4705-b66e-9f83f322deb9}">
        <Seed>123412354</Seed>
        <MovingFactor>1</MovingFactor>
        </Application>
        </Applications>
        </Token>
        
        <Token serial="00040008CFA52">
        <CaseModel>5</CaseModel>
        <Model>101</Model>
        <ProductionDate>02/19/2009</ProductionDate>
        <ProductName>Safeword Alpine</ProductName>
        <Applications>
        <Application ConnectorID="{ab1397d2-ddb6-4705-b66e-9f83f322deb9}">
        <Seed>123456</Seed>
        <MovingFactor>1</MovingFactor>
        </Application>
        </Applications>
        </Token>
        
        </Tokens>

        '''
        TOKENS = privacyidea.lib.ImportOTP.parseSafeNetXML(xml)

        assert len(TOKENS) == 2



    def test_parse_OATH(self):
        '''
        Test an OATH csv import
        '''
        csv = '''
        tok1, 1212
        tok2, 1212, totp, 6
        tok3, 1212, hotp, 8
        tok4, 1212, totp, 8, 60
        '''
        TOKENS = privacyidea.lib.ImportOTP.parseOATHcsv(csv)

        assert len(TOKENS) == 4

        assert TOKENS["tok4"].get("timeStep") == 60

        assert TOKENS["tok3"].get("otplen") == 8

    def test_import_OATH(self):
        '''
        test to import token data
        '''
        csv = '''
        tok1, 1212
        tok2, 1212, totp, 6
        tok3, 1212, hotp, 8
        tok4, 1212, totp, 8, 60
        '''

        response = self.app.post(url(controller='admin', action='loadtokens'), params={'file':csv, 'type':'oathcsv'})

        assert '"imported": 4' in response

    def test_import_PSKC(self):
        '''
        Test to import PSKC data
        '''
        #FIXME EE import
        return
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':XML_PSKC,
                                         'type':'pskc',
                                         'pskc_type': 'plain',
                                         'pskc_password': "",
                                         'pskc_preshared': ""})
        print response
        assert '"imported": 6' in response

        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':XML_PSKC,
                                         'type':'pskc',
                                         'pskc_type': 'plain',
                                         'pskc_password': "",
                                         'pskc_preshared': "",
                                         'pskc_checkserial': 'true'})
        print response
        assert '"imported": 0' in response

    def test_import_empty_file(self):
        '''
        Test loading empty file
        '''
        #FIXME EE import
        return
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':"",
                                         'type':'pskc',
                                         'pskc_type': 'plain',
                                         'pskc_password': "",
                                         'pskc_preshared': ""})
        print response
        assert '"status": false' in response
        assert '"message": "Error loading tokens. File or Type empty!",' in response

    def test_import_unknown(self):
        '''
        Test to import unknown type
        '''
        response = self.app.post(url(controller='admin', action='loadtokens'),
                                 params={'file':XML_PSKC,
                                         'type':'XYZ'})
        print response
        assert '"status": false' in response
        assert 'Unknown file type' in response

    def test_import_XML(self):
        '''
        Test to import XML data
        '''
        xmls = '''
        <Tokens>
        <Token serial="00040008CFA5">
        <CaseModel>5</CaseModel>
        <Model>101</Model>
        <ProductionDate>02/19/2009</ProductionDate>
        <ProductName>Safeword Alpine</ProductName>
        <Applications>
        <Application ConnectorID="{ab1397d2-ddb6-4705-b66e-9f83f322deb9}">
        <Seed>123412354</Seed>
        <MovingFactor>1</MovingFactor>
        </Application>
        </Applications>
        </Token>
        
        <Token serial="00040008CFA52">
        <CaseModel>5</CaseModel>
        <Model>101</Model>
        <ProductionDate>02/19/2009</ProductionDate>
        <ProductName>Safeword Alpine</ProductName>
        <Applications>
        <Application ConnectorID="{ab1397d2-ddb6-4705-b66e-9f83f322deb9}">
        <Seed>123456</Seed>
        <MovingFactor>1</MovingFactor>
        </Application>
        </Applications>
        </Token>
        
        </Tokens>
        '''
        response = self.app.post(url(controller='admin', action='loadtokens'), params={'file':xmls, 'type':'aladdin-xml'})
        assert '"imported": 2' in response
        return

    def test_import_Yubikey(self):
        '''
        Test to import Yubikey CSV
        '''
        csv='''
        Static Password: Scan Code,17.04.12 12:25,1,051212172c092728,,,,,0,0,0,0,0,0,0,0,0,0
        Static Password: Scan Code,17.04.12 12:27,1,282828051212172c092728,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,17.04.12 12:29
        Static Password: Scan Code,17.04.12 12:29,1,2828282828051212172c092728,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 16:43
        Yubico OTP,11.12.13 16:43,1,cccccccirblh,b321173a2fb8,6faa3ce885fcd5eda7efa5195e5a5d44,,,0,0,0,0,0,0,0,0,0,1
        Yubico OTP,11.12.13 16:43,1,ccccccbgbhkl,9b19889fc5c1,11261596dbbeae6538b26ce0cfd4f9c9,,,0,0,0,0,0,0,0,0,0,1
        LOGGING START,11.12.13 18:55
        OATH-HOTP,11.12.13 18:55,1,cccccccccccc,,916821d3a138bf855e70069605559a206ba854cd,,,0,0,0,6,0,0,0,0,0,0
        LOGGING START,11.12.13 18:58
        Yubico OTP,11.12.13 18:58,1,,,a54c68c7c3d5a1fec8a0c85b8d60765b,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 19:00
        OATH-HOTP,11.12.13 19:00,1,cccccccccccc,,1390612c06ec6dd0fa077ce99bf9c86d2c058f42,,,0,0,0,6,0,0,0,0,0,0
        LOGGING START,11.12.13 19:01
        OATH-HOTP,11.12.13 19:01,1,,,d41845578effd750887edc70f04df754603e2b63,,,0,0,0,6,0,0,0,0,0,0
        LOGGING START,11.12.13 19:05
        Static Password: Scan Code,11.12.13 19:05,1,040716040416070416041607041607,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 19:06
        Static Password: Scan Code,11.12.13 19:06,1,1e1f201a1407048796,,,,,0,0,0,0,0,0,0,0,0,0
        Static Password: Scan Code,11.12.13 19:06,1,1e1f201a1407048796,,,,,0,0,0,0,0,0,0,0,0,0
        LOGGING START,11.12.13 19:07
        Static Password,11.12.13 19:07,1,,ba23877e747c,fe8abdf8c0c9b6ad6a1daabefa4d50b3,,,0,0,0,0,0,0,0,0,0,0
        Static Password,11.12.13 19:08,1,,d5a3d50327dc,0e8e37b0e38b314a56748c030f58d21d,,,0,0,0,0,0,0,0,0,0,0
        '''
        response = self.app.post(url(controller='admin', action='loadtokens'), params={'file':csv, 'type':'yubikeycsv'})
        print response
        assert '"imported": 3' in response
        return



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''    
  Description:  This file contains the definition of the simple pass token class
  
  Dependencies: -

'''

import logging

from privacyidea.lib.util    import getParam
from privacyidea.lib.validate import check_pin
from privacyidea.lib.log import log_with

optional = True
required = False

from privacyidea.lib.tokenclass import TokenClass

log = logging.getLogger(__name__)




class SpassTokenClass(TokenClass):
    '''
    This is a simple pass token.
    It does have no OTP component. The OTP checking will always
    succeed. Of course, an OTP PIN can be used.
    '''
    def __init__(self, aToken):
        TokenClass.__init__(self, aToken)
        self.setType(u"spass")
        self.mode = ['authenticate']

    @classmethod
    def getClassType(cls):
        return "spass"

    @classmethod
    def getClassPrefix(cls):
        return "LSSP"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        '''
        res = {
               'type'           : 'spass',
               'title'          : 'Simple Pass Token',
               'description'    : ('A token that allows the user to simply pass. Can be combined with the OTP PIN.'),
               'init'         : {'page' : {'html'      : 'spasstoken.mako',
                                            'scope'      : 'enroll', },
                                   'title'  : {'html'      : 'spasstoken.mako',
                                             'scope'     : 'enroll.title', },
                                   },
               'config'        : {},
               'selfservice'   :  {},
               'policy' : {},
               }

        # do we need to define the lost token policies here...
        if key is not None and res.has_key(key):
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret

    def update(self, param):
        # cko: changed for backward compat
        getParam(param, "pin", optional)
        if not param.has_key('otpkey'):
            param['genkey'] = 1

        TokenClass.update(self, param)

    ## the spass token does not suport challenge response
    def is_challenge_request(self, passw, user, options=None):
        return False

    def is_challenge_response(self, passw, user, options=None, challenges=None):
        return False

    @log_with(log)
    def authenticate(self, passw, user, options=None):
        '''
        in case of a wrong passw, we return a bad matching pin,
        so the result will be an invalid token
        '''
        otp_count = -1
        pin_match = check_pin(self, passw, user=user, options=options)
        if pin_match == True:
            otp_count = 0
        return (pin_match, otp_count, None)

## eof ########################################################################




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel, info@privacyidea.org
#  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""SQLAlchemy Metadata and Session object"""
from sqlalchemy import MetaData
from sqlalchemy.orm import scoped_session, sessionmaker

__all__ = ['Session', 'engine', 'metadata']

# SQLAlchemy database engine. Updated by model.init_model()
engine = None

# SQLAlchemy session manager. Updated by model.init_model()
Session = scoped_session(sessionmaker(autocommit=False, autoflush=True))

# Global metadata. If you have multiple databases with overlapping table
# names, you'll need a metadata for each database
metadata = MetaData()



'''
The useridresolver is responsible for getting userids for loginnames and vice versa.

This base module contains the base class UserIdResolver.UserIdResolver and also the
community class PasswdIdResolver.IdResolver, that is inherited from the base class.
'''



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.880529
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/motptoken.mako'
_template_uri = '/motptoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n')
        # SOURCE LINE 4
        if c.scope == 'config.title' :
            # SOURCE LINE 5
            __M_writer(u' ')
            __M_writer(escape(_("mOTP Token Settings")))
            __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n\n')
        # SOURCE LINE 9
        if c.scope == 'config' :
            pass
        # SOURCE LINE 11
        __M_writer(u'\n\n')
        # SOURCE LINE 13
        if c.scope == 'enroll.title' :
            # SOURCE LINE 14
            __M_writer(escape(_("mOTP - mobile otp")))
            __M_writer(u'\n')
        # SOURCE LINE 16
        __M_writer(u'\n')
        # SOURCE LINE 17
        if c.scope == 'enroll' :
            # SOURCE LINE 18
            __M_writer(u'<script>\n\n/*\n * \'typ\'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\n\nfunction motp_get_enroll_params(){\n    var url = {};\n    url[\'type\'] = \'motp\';\n   \turl[\'description\'] = $(\'#enroll_motp_desc\').val();\n\n    // If we have got to generate the hmac key, we do it here:\n    if  ( $(\'#motp_key_cb\').attr(\'checked\') ) {\n    \turl[\'genkey\'] = 1;\n\n    } else {\n        // OTP Key\n        url[\'otpkey\'] \t= $(\'#motp_initsecret\').val();\n    }   \t\n   \t\n\turl[\'otppin\']\t= $(\'#motp_pin1\').val();\n    jQuery.extend(url, add_user_data());\n\n    return url;\n}\n</script>\n\n<p><span id="motp_key_intro">\n\t')
            # SOURCE LINE 50
            __M_writer(escape(_("Please enter or copy the init-secret, that was generated by your app.")))
            __M_writer(u'\n</span>\n</p>\n<table><tr>\n<td><label for="motp_initsecret" id=\'motp_initsecret_label\'>')
            # SOURCE LINE 54
            __M_writer(escape(_("Init secret")))
            __M_writer(u'</label></td>\n<td><input type="text" name="motp_initsecret" id="motp_initsecret"\n\ttitle=\'')
            # SOURCE LINE 56
            __M_writer(escape(_("Please enter or copy the init-secret, that was generated by your app.")))
            __M_writer(u'\' \n\tvalue="" class="text ui-widget-content ui-corner-all" /></td>\n</tr><tr>\n\t<td> </td><td><input type=\'checkbox\' id=\'motp_key_cb\' onclick="cb_changed(\'motp_key_cb\',[\'motp_initsecret\',\'motp_initsecret_label\',\'motp_key_intro\']);">\n\t<label for=hmac_key_cb>')
            # SOURCE LINE 60
            __M_writer(escape(_("Generate MOTP key.")))
            __M_writer(u'</label></td>\n</tr>\n<tr><td colspan="2">')
            # SOURCE LINE 62
            __M_writer(escape(_("Also enter the PIN you are using on your phone.")))
            __M_writer(u'</td></tr>\n<tr>\n<td><label for="motp_pin1">mOTP PIN</label></td>\n<td><input onkeyup="checkpins(\'motp_pin1\',\'motp_pin2\');" autocomplete="off"\n     title=\'')
            # SOURCE LINE 66
            __M_writer(escape(_("Also enter the PIN you are using on your phone.")))
            __M_writer(u'\' \n     type="password" name="motp_pin1" id="motp_pin1" value="" class="text ui-widget-content ui-corner-all" /></td>\n</tr><tr>\n<td><label for="motp_pin2">')
            # SOURCE LINE 69
            __M_writer(escape(_("(again)")))
            __M_writer(u'</label></td>\n<td><input onkeyup="checkpins(\'motp_pin1\',\'motp_pin2\');" autocomplete="off" type="password" name="motp_pin2" id="motp_pin2" value="" class="text ui-widget-content ui-corner-all" /></td>\n</tr>\n<tr>\n    <td><label for="enroll_motp_desc" id=\'enroll_motp_desc_label\'>')
            # SOURCE LINE 73
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n    <td><input type="text" name="enroll_motp_desc" id="enroll_motp_desc" value="webGUI_generated" class="text" /></td>\n</tr>\n</table>\n\n')
        # SOURCE LINE 79
        __M_writer(u'\n\n\n\n')
        # SOURCE LINE 83
        if c.scope == 'selfservice.title.enroll':
            # SOURCE LINE 84
            __M_writer(escape(_("Register mOTP")))
            __M_writer(u'\n')
        # SOURCE LINE 86
        __M_writer(u'\n')
        # SOURCE LINE 87
        if c.scope == 'selfservice.title.provision':
            # SOURCE LINE 88
            __M_writer(escape(_("Webprovisioning mOTP")))
            __M_writer(u'\n')
        # SOURCE LINE 90
        __M_writer(u'\n\n')
        # SOURCE LINE 92
        if c.scope == 'selfservice.enroll':
            # SOURCE LINE 93
            __M_writer(u'<script>\n\tjQuery.extend(jQuery.validator.messages, {\n\t\trequired:  "')
            # SOURCE LINE 95
            __M_writer(escape(_('required input field')))
            __M_writer(u'",\n\t\tminlength: "')
            # SOURCE LINE 96
            __M_writer(escape(_('minimum length must be greater than {0}')))
            __M_writer(u'",\n\t\tmaxlength: "')
            # SOURCE LINE 97
            __M_writer(escape(_('maximum length must be lower than {0}')))
            __M_writer(u'",\n\t\trange: \'')
            # SOURCE LINE 98
            __M_writer(escape(_("Please enter a valid init secret. It may only contain numbers and the letters A-F.")))
            __M_writer(u'\',\n\t});\n\n\tjQuery.validator.addMethod("motp_secret", function(value, element, param){\n\t\tif ( $(\'#motp_key_cb\').attr(\'checked\') ) {\n\t\t\treturn true\n\t\t} else {\n\t\t\tif (value.length < 16) {\n\t\t\t\treturn false\n\t\t\t}\n        \treturn value.match(/^[a-fA-F0-9]+$/i);\n\t\t}\n    }, \'')
            # SOURCE LINE 110
            __M_writer(escape(_("Please enter a valid init secret. It may only contain numbers and the letters A-F.")))
            __M_writer(u'\' );\n\n\t$(\'#form_registermotp\').validate({\n        rules: {\n            motp_secret: {\n                minlength: 16,\n                maxlength: 32,\n                number: false,\n                motp_secret: true\n            }\n        }\n\t});\n\nfunction self_motp_get_param()\n{\n\tvar urlparam = {};\n\n\turlparam[\'type\'] \t\t= \'motp\';\n\turlparam[\'description\'] = $("#motp_self_desc").val();  \n    urlparam[\'otpkey\'] \t= $(\'#motp_secret\').val();\n\turlparam[\'otppin\']\t\t= $(\'#motp_s_pin1\').val();\n\n\treturn urlparam;\n}\nfunction self_motp_clear()\n{\n\t$(\'#motp_s_pin1\').val(\'\');\n\t$(\'#motp_secret\').val(\'\');\n\t$(\'#motp_s_pin2\').val(\'\');\n}\nfunction self_motp_submit(){\n\n\tvar ret = false;\n\tif ($(\'#form_registermotp\').valid()) {\n\t\tvar params =  self_motp_get_param();\n\t\tenroll_token( params );\n\t\tself_motp_clear();\n\t\tret = true;\n\t} else {\n\t\talert(\'')
            # SOURCE LINE 149
            __M_writer(escape(_("Form data not valid.")))
            __M_writer(u"');\n\t}\n\treturn ret;\n\n}\n</script>\n\n<h1>")
            # SOURCE LINE 156
            __M_writer(escape(_("Register your mOTP Token")))
            __M_writer(u'</h1>\n<div id=\'registermotpform\'>\n\t<form class="cmxform" id=\'form_registermotp\'>\n\t<fieldset>\n\t\t<table>\n\t\t<tr>\n\t\t<td><label for=\'secret\' id=\'motp_secret_label\'>')
            # SOURCE LINE 162
            __M_writer(escape(_("Init Secret of motp-Token")))
            __M_writer(u'</label></td>\n\t\t<td><input id=\'motp_secret\' name=\'secret\' class="ui-widget-content ui-corner-all"\n\t\t\tpattern="[a-fA-F0-9]{0,16,32}"\n\t\t\tmaxlength="32" />\n\t\t</td>\n\t\t</tr>\n        <tr>\n        <td><label for=\'motp_s_pin1\'>')
            # SOURCE LINE 169
            __M_writer(escape(_("mOTP PIN")))
            __M_writer(u'</label></td>\n        <td><input autocomplete="off" type="password" onkeyup="checkpins(\'motp_s_pin1\', \'motp_s_pin2\');" id="motp_s_pin1" class="required text ui-widget-content ui-corner-all" /></td>\n        </tr>\n        <tr>\n        <td><label for=\'motp_s_pin2\'>')
            # SOURCE LINE 173
            __M_writer(escape(_("mOTP PIN (again)")))
            __M_writer(u'</label></td>\n        <td><input autocomplete="off" type="password" onkeyup="checkpins(\'motp_s_pin1\', \'motp_s_pin2\');" id="motp_s_pin2" class="required text ui-widget-content ui-corner-all" /></td>\n        </tr>\n\t\t<tr>\n\t    <td><label for="motp_self_desc" id=\'motp_self_desc_label\'>')
            # SOURCE LINE 177
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n\t    <td><input type="text" name="motp_self_desc" id="motp_self_desc" value="self enrolled" class="text" /></td>\n\t\t</tr>\n        </table>\n        <button class=\'action-button\' id=\'button_register_motp\'\n        \t\tonclick="self_motp_submit();">')
            # SOURCE LINE 182
            __M_writer(escape(_("register mOTP Token")))
            __M_writer(u'</button>\n    </fieldset>\n    </form>\n</div>\n\n\n')
        # SOURCE LINE 189
        __M_writer(u'\n\n')
        # SOURCE LINE 191
        if c.scope == 'selfservice.provision':
            # SOURCE LINE 192
            __M_writer(u'<script>\n\nfunction self_motp_provision_get_param()\n{\n\tvar urlparam = {};\n\n\turlparam[\'type\'] \t\t= \'motp\';\n\turlparam[\'description\'] = $("#motpP_self_desc").val();  \n   \turlparam[\'genkey\'] = 1;\n\turlparam[\'otppin\']\t\t= $(\'#motpP_s_pin1\').val();\n\n\treturn urlparam;\n}\nfunction self_motp_provision_clear()\n{\n\t$(\'#motpP_s_pin1\').val(\'\');\n\t$(\'#motpP_s_pin2\').val(\'\');\n}\nfunction self_motp_provision_submit(){\n\n\tvar ret = false;\n\tif ($(\'#form_provisionmotp\').valid()) {\n\t\tvar params =  self_motp_provision_get_param();\n\t\tenroll_token( params );\n\t\tself_motp_provision_clear();\n\t\tret = true;\n\t} else {\n\t\talert(\'')
            # SOURCE LINE 219
            __M_writer(escape(_("Form data not valid.")))
            __M_writer(u"');\n\t}\n\treturn ret;\n\n}\n</script>\n\n<h1>")
            # SOURCE LINE 226
            __M_writer(escape(_("Webprovisioning your mOTP Token")))
            __M_writer(u'</h1>\n<div id=\'provisionmotpform\'>\n\t<form class="cmxform" id=\'form_provisionmotp\'>\n\t<fieldset>\n\t\t<table>\n\t\t<tr> <td colspan=2>\n\t\t')
            # SOURCE LINE 232
            __M_writer(escape(_("Your mOTP secret will be created by the server.")))
            __M_writer(u"\n\t\t</td>\n\t\t</tr>\n        <tr>\n        <td><label for='motpP_s_pin1'>")
            # SOURCE LINE 236
            __M_writer(escape(_("mOTP PIN")))
            __M_writer(u'</label></td>\n        <td><input autocomplete="off" type="password" \n        \tonkeyup="checkpins(\'motpP_s_pin1\', \'motpP_s_pin2\');" \n        \tid="motpP_s_pin1" \n        \tclass="required text ui-widget-content ui-corner-all" /></td>\n        </tr>\n        <tr>\n        <td><label for=\'motpP_s_pin2\'>')
            # SOURCE LINE 243
            __M_writer(escape(_("mOTP PIN (again)")))
            __M_writer(u'</label></td>\n        <td><input autocomplete="off" type="password"\n        \tonkeyup="checkpins(\'motpP_s_pin1\', \'motpP_s_pin2\');"\n        \tid="motpP_s_pin2"\n        \tclass="required text ui-widget-content ui-corner-all" /></td>\n        </tr>\n\t\t<tr>\n\t    <td><label for="motpP_self_desc"\n\t    \tid=\'motpP_self_desc_label\'>')
            # SOURCE LINE 251
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n\t    <td><input type="text"\n\t    \tname="motpP_self_desc"\n\t    \tid="motpP_self_desc"\n\t    \tvalue="self enrolled" class="text" /></td>\n\t\t</tr>\n        </table>\n        <button class=\'action-button\'\n        \t\tid=\'button_provision_motp\'\n        \t\tonclick="self_motp_provision_submit();">')
            # SOURCE LINE 260
            __M_writer(escape(_("provision mOTP Token")))
            __M_writer(u'</button>\n    </fieldset>\n    </form>\n</div>\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#
#    privacyIDEA is a fork of LinOTP
#    May 28, 2014 Cornelius Kölbel
#    E-mail: info@privacyidea.org
#    Contact: www.privacyidea.org
#
#    Copyright (C) LinOTP: 2010 - 2014 LSE Leading Security Experts GmbH
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#

"""
This is the SMSClass to send SMS via SMTP Gateway.
i.e. a Mail is sent to an Gateway/Emailserver and dependig on the
address, subject and body this gateway will trigger the sending of the SMS. 
"""

from privacyidea.smsprovider.SMSProvider import ISMSProvider

import string
import smtplib

import logging
log = logging.getLogger(__name__)

PHONE_TAG = "<phone>"
MSG_TAG = "<otp>"

class SmtpSMSProvider(ISMSProvider):
    def __init__(self):
        self.config = {}

    '''
      submitMessage()
      - send out a message to a phone

    '''


    def submitMessage(self, phone, message, exception=True):
        '''
        Submits the message for phone to the email gateway.

        Returns true in case of success

        Remarks:
        the exception parameter is not in the official interface and
        the std handling is to pass the exception up to the upper levels.

        TODO: fix the interface w.r.t. the exception parameter

        '''
        ret = False
        if (not self.config.has_key('mailserver')
            or not self.config.has_key('mailsender')
            or not self.config.has_key('mailto')):
            log.error("[submitMessage] incomplete config: %s. mailserver, mailsender and mailto needed." % self.config)
            return ret

        server = self.config.get("mailserver")
        user = self.config.get("mailuser")
        password = self.config.get("mailpassword")
        fromaddr = self.config.get("mailsender", "privacyidea@localhost")
        toaddr = self.config.get("mailto")
        subject = self.config.get("subject", "")
        body = self.config.get("body", "")

        log.debug("[submitMessage] submitting message %s to %s" % (message, phone))

        toaddr = string.replace(toaddr, PHONE_TAG, phone)

        subject = string.replace(subject, PHONE_TAG, phone)
        subject = string.replace(subject, MSG_TAG, message)

        body = string.replace(body, PHONE_TAG, phone)
        body = string.replace(body, MSG_TAG, message)

        msg = ("From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n%s"
               % (fromaddr,
                  toaddr,
                  subject,
                  body))

        try:
            serv = smtplib.SMTP(server)
            serv.set_debuglevel(1)
            if user:
                log.debug("authenticating to mailserver, user: %s, pass: %s" % (user, password))
                serv.login(user, password)
            dict = serv.sendmail(fromaddr, toaddr, msg)
            #print "sendmail::: ", dict
            (code, response) = serv.quit()
            #print "quit::: ", code, response
            ret = True
        except Exception as  e:
            #print "[submitMessage] %s" % str(e)
            log.error("[submitMessage] %s" % str(e))
            if exception:
                raise Exception(e)
            ret = False

        return ret

    def loadConfig(self, configDict):
        self.config = configDict




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file contains the definition of the RADIUS token class
  
  Dependencies: -

'''

import logging

import traceback
import binascii

from privacyidea.lib.util    import getParam
from privacyidea.lib.log import log_with

optional = True
required = False

## for update, we require the TokenClass
from privacyidea.lib.tokenclass import TokenClass
from privacyidea.lib.tokens.remotetoken import RemoteTokenClass


log = logging.getLogger(__name__)

# we need this for the radius token
import pyrad.packet
from pyrad.client import Client
from pyrad.dictionary import Dictionary
from pylons.configuration import config as env


VOID_RADIUS_SECRET = "voidRadiusSecret"

###############################################
class RadiusTokenClass(RemoteTokenClass):

    def __init__(self, aToken):
        RemoteTokenClass.__init__(self, aToken)
        self.setType(u"radius")

        self.radiusServer = ""
        self.radiusUser = ""
        self.radiusLocal_checkpin = "0"
        self.radiusSecret = VOID_RADIUS_SECRET

    @classmethod
    def getClassType(cls):
        return "radius"

    @classmethod
    def getClassPrefix(cls):
        return "LSRA"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        '''
        res = {
               'type'           : 'radius',
               'title'          : 'RADIUS Token',
               'description'    : ('RADIUS token to forward the authentication request to another RADIUS server'),

               'init'         : {'page' : {'html'      : 'radiustoken.mako',
                                            'scope'      : 'enroll', },
                                   'title'  : {'html'      : 'radiustoken.mako',
                                             'scope'     : 'enroll.title', },
                                   },

               'config'        : { 'page' : {'html'      : 'radiustoken.mako',
                                            'scope'      : 'config', },
                                   'title'  : {'html'      : 'radiustoken.mako',
                                             'scope'     : 'config.title', },
                                 },

               'selfservice'   :  {},
               'policy' : {},
               }

        if key is not None and res.has_key(key):
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret


    def update(self, param):

        self.radiusServer = getParam(param, "radius.server", required)
        # if another OTP length would be specified in /admin/init this would
        # be overwritten by the parent class, which is ok.
        self.setOtpLen(6)

        val = getParam(param, "radius.local_checkpin", optional)
        if val is not None:
            self.radiusLocal_checkpin = val

        val = getParam(param, "radius.user", required)
        if val is not None:
            self.radiusUser = val

        val = getParam(param, "radius.secret", required)
        if val is not None:
            self.radiusSecret = val

        if self.radiusSecret == VOID_RADIUS_SECRET:
            log.warning("Usage of default radius secret is not recomended!!")


        TokenClass.update(self, param)
        # We need to write the secret!
        self.token.setHKey(binascii.hexlify(self.radiusSecret))

        self.addToTokenInfo("radius.server", self.radiusServer)
        self.addToTokenInfo("radius.local_checkpin", self.radiusLocal_checkpin)
        self.addToTokenInfo("radius.user", self.radiusUser)

    @log_with(log)
    def check_pin_local(self):
        """
        lookup if pin should be checked locally or on radius host

        :return: bool
        """
        local_check = False

        if 1 == int(self.getFromTokenInfo("radius.local_checkpin")):
            local_check = True
        log.debug(" local checking pin? %r" % local_check)

        return local_check

    @log_with(log)
    def checkPin(self, pin, options=None):
        '''
        check the pin - either remote or localy
        - in case of remote, we return true, as the
          the splitPinPass will put the passw then in the otpVal

        :param pin: the pin which should be checked
        :param options: the additional request parameters
        '''
        res = True

        if self.check_pin_local():
            log.debug("checking PIN locally")
            res = RemoteTokenClass.checkPin(self, pin)

        return res

    @log_with(log)
    def splitPinPass(self, passw):
        '''
        Split the PIN and the OTP value.
        Only if it is locally checked and not remotely.
        '''
        res = 0
        pin = ""
        otpval = ""

        local_check = self.check_pin_local()
        log.debug("local checking pin? %r"
                  % local_check)

        if self.check_pin_local():
            log.debug("locally checked")
            (res, pin, otpval) = TokenClass.splitPinPass(self, passw)
        else:
            log.debug("remotely checked")
            pin = ""
            otpval = passw
        return (res, pin, otpval)

    @log_with(log)
    def do_request(self, anOtpVal, transactionid=None, user=None):
        '''
        Here we contact the Radius Server to verify the pass
        '''
        log.debug("do_request")

        reply = {}
        res = False
        otp_count = -1

        radiusServer = self.getFromTokenInfo("radius.server")
        radiusUser = self.getFromTokenInfo("radius.user")

        ## Read the secret!!!
        secret = self.token.getHOtpKey()
        radiusSecret = binascii.unhexlify(secret.getKey())

        if radiusSecret == VOID_RADIUS_SECRET:
            log.warning("Usage of default radius secret is not recomended!!")

        ## here we also need to check for radius.user
        log.debug("checking OTP len:%s on radius server: %s, user: %s" 
                  % (len(anOtpVal), radiusServer, radiusUser))

        try:
            # pyrad does not allow to set timeout and retries.
            # it defaults to retries=3, timeout=5

            # TODO: At the moment we support only one radius server.
            # No round robin.
            server = radiusServer.split(':')
            r_server = server[0]
            r_authport = 1812
            nas_identifier = env.get("radius.nas_identifier", "privacyIDEA")
            r_dict = env.get("radius.dictfile", "/etc/privacyidea/dictionary")

            if len(server) >= 2:
                r_authport = int(server[1])
            log.debug("NAS Identifier: %r, "
                      "Dictionary: %r" % (nas_identifier, r_dict))

            log.debug("constructing client object "
                      "with server: %r, port: %r, secret: %r" %
                      (r_server, r_authport, radiusSecret))

            srv = Client(server=r_server,
                       authport=r_authport,
                       secret=radiusSecret,
                       dict=Dictionary(r_dict))

            req = srv.CreateAuthPacket(code=pyrad.packet.AccessRequest,
                                       User_Name=radiusUser.encode('ascii'),
                                       NAS_Identifier=nas_identifier.encode('ascii'))

            req["User-Password"] = req.PwCrypt(anOtpVal)
            if transactionid is not None:
                req["State"] = str(transactionid)

            response = srv.SendPacket(req)

            if response.code == pyrad.packet.AccessChallenge:
                opt = {}
                for attr in response.keys():
                    opt[attr] = response[attr]
                res = False
                log.debug("challenge returned %r " % opt)
                ## now we map this to a privacyidea challenge
                if "State" in opt:
                    reply["transactionid"] = opt["State"][0]
                if "Reply-Message" in opt:
                    reply["message"] = opt["Reply-Message"][0]

            elif response.code == pyrad.packet.AccessAccept:
                log.info("Radiusserver %s granted "
                         "access to user %s." % (r_server, radiusUser))
                otp_count = 0
                res = True
            else:
                log.warning("Radiusserver %s"
                            "rejected access to user %s." %
                            (r_server, radiusUser))
                res = False

        except Exception as ex:
            log.error("Error contacting radius Server: %r" % (ex))
            log.error(traceback.format_exc())

        return (res, otp_count, reply)




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
from logging import Formatter
import string
import logging
import functools
log = logging.getLogger(__name__)

class SecureFormatter(Formatter):

    bad_chars = "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19"

    def format(self, record):
        try:
            message = super(SecureFormatter, self).format(record)
        except TypeError:
            # In pyhton 2.6 the Formatter does not seem to 
            # be defined as 
            # class Formatter(object)
            # Using it in the super-statement this will raise a TypeError
            message = Formatter.format(self, record)
        secured = False

        s = ""
        for c in message:
            if c in string.printable:
                s += c
            else:
                s += '.'
                secured = True

        if secured:
            s = "!!!Log Entry Secured by SecureFormatter!!! " + s

        return s


class log_with(object):
    '''Logging decorator that allows you to log with a
    specific logger.
    '''
    # Customize these messages
    ENTRY_MESSAGE = u'Entering {0} with arguments {1} and keywords {2}'
    EXIT_MESSAGE = 'Exiting {0} with result {1}'
    
    def __init__(self, logger=None, log_entry=True, log_exit=True):
        self.logger = logger
        self.log_exit = log_exit
        self.log_entry = log_entry

    def __call__(self, func):
        '''Returns a wrapper that wraps func.
        The wrapper will log the entry and exit points of the function
        with logging.INFO level.
        '''
        # set logger if it was not set earlier
        if not self.logger:
            logging.basicConfig()
            self.logger = logging.getLogger(func.__module__)
            
        @functools.wraps(func)
        def wrapper(*args, **kwds):
            try:
                if self.log_entry:
                    self.logger.debug(self.ENTRY_MESSAGE.format(func.__name__, args, kwds))
                else:
                    self.logger.debug(self.ENTRY_MESSAGE.format(func.__name__, "HIDDEN", "HIDDEN"))
            except Exception as exx:
                self.logger.error("Error during logging of function {0}! {1}".format(func.__name__, exx))
                
            f_result = func(*args, **kwds)
            
            try:
                if self.log_exit:
                    self.logger.debug(self.EXIT_MESSAGE.format(func.__name__, f_result))
                else:
                    self.logger.debug(self.EXIT_MESSAGE.format(func.__name__, "HIDDEN"))
            except Exception as exx:
                self.logger.error("Error during logging of function {0}! {1}".format(func.__name__, exx))
            return f_result
        
        return wrapper



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.839633
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/remotetoken.mako'
_template_uri = '/remotetoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        Exception = context.get('Exception', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n')
        # SOURCE LINE 4
        if c.scope == 'config.title' :
            # SOURCE LINE 5
            __M_writer(u' ')
            __M_writer(escape(_("Remote Token")))
            __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n\n')
        # SOURCE LINE 9
        if c.scope == 'config' :
            # SOURCE LINE 10
            __M_writer(u'<script>\n\n/*\n * \'typ\'_get_config_val()\n *\n * this method is called, when the token config dialog is opened\n * - it contains the mapping of config entries to the form id\n * - according to the Config entries, the form entries will be filled\n *\n */\n\n\nfunction remote_get_config_val(){\n\tvar id_map = {};\n\n    id_map[\'remote.server\'] \t\t= \'sys_remote_server\';\n    id_map[\'remote.local_checkpin\'] = \'sys_remote_local_checkpin\';\n    id_map[\'remote.realm\'] \t\t\t= \'sys_remote_realm\';\n    id_map[\'remote.resConf\'] \t\t= \'sys_remote_resConf\';\n\n    // FIXME: We need to set the checkpin select box. Do not know how!\n\n\treturn id_map;\n\n}\n\n/*\n * \'typ\'_get_config_params()\n *\n * this method is called, when the token config is submitted\n * - it will return a hash of parameters for system/setConfig call\n *\n */\nfunction remote_get_config_params(){\n\tvar url_params ={};\n\n    url_params[\'remote.server\'] \t= $(\'#sys_remote_server\').val();\n    url_params[\'remote.realm\'] \t= $(\'#sys_remote_realm\').val();\n    url_params[\'remote.resConf\'] \t= $(\'#sys_remote_resConf\').val();\n    url_params[\'remote.remote_checkpin\'] \t= $(\'#sys_remote_local_checkpin\').val();\n\n\treturn url_params;\n}\n\n\njQuery.validator.addMethod("sys_remote_server", function(value, element, param){\n      return value.match(param);\n}, "')
            # SOURCE LINE 57
            __M_writer(escape(_('Please enter a valid remote server specification. It needs to be of the form http://server or https://server')))
            __M_writer(u'");\n\n\n$("#form_config_remote").validate({\n         rules: {\n            sys_remote_server: {\n                required: true,\n                number: false,\n                \tsys_remote_server: /^(http:\\/\\/|https:\\/\\/)/i\n             }\n         }\n     });\n\n</script>\n\n<form class="cmxform" id=\'form_config_remote\'>\n\t<table>\n\t<tr>\n\t<td><label for="sys_remote_server" title=\'')
            # SOURCE LINE 75
            __M_writer(escape(_("You need to enter the remote privacyIDEA server like https://remoteprivacyidea")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 76
            __M_writer(escape(_("REMOTE server")))
            __M_writer(u'</label></td>\n\t<td><input class="required" type="text" name="sys_remote_server" id="sys_remote_server"\n\t\tclass="text ui-widget-content ui-corner-all" value="https://localhost"/></td>\n\t</tr>\n\n\t<tr><td><label for="sys_remote_local_checkpin" title=\'')
            # SOURCE LINE 81
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or forwarded to the remote server")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 82
            __M_writer(escape(_("check PIN")))
            __M_writer(u'</label></td>\n\t<td><select name="sys_remote_local_checkpin" id="sys_remote_local_checkpin"\n\t\ttitle=\'')
            # SOURCE LINE 84
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or on the remote server")))
            __M_writer(u"'>\n\t\t\t<option value=0>")
            # SOURCE LINE 85
            __M_writer(escape(_("on REMOTE server")))
            __M_writer(u'</option>\n\t\t\t<option value=1>')
            # SOURCE LINE 86
            __M_writer(escape(_("locally")))
            __M_writer(u'</option>\n\t\t</select></td>\n\t</tr>\n\n\t<tr>\n\t<td><label for="sys_remote_realm">')
            # SOURCE LINE 91
            __M_writer(escape(_("Remote Realm")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="sys_remote_realm" id="sys_remote_realm"\n\t\tclass="text ui-widget-content ui-corner-all" /></td>\n\t</tr>\n\n\t<tr>\n\t<td><label for="sys_remote_resConf">')
            # SOURCE LINE 97
            __M_writer(escape(_("Remote Resolver")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="sys_remote_resConf" id="sys_remote_resConf"\n\t\tclass="text ui-widget-content ui-corner-all" /></td>\n\t</tr>\n\t</table>\n\n</form>\n')
        # SOURCE LINE 105
        __M_writer(u'\n\n')
        # SOURCE LINE 107
        if c.scope == 'enroll.title' :
            # SOURCE LINE 108
            __M_writer(escape(_("REMOTE token")))
            __M_writer(u'\n')
        # SOURCE LINE 110
        __M_writer(u'\n')
        # SOURCE LINE 111
        if c.scope == 'enroll' :
            # SOURCE LINE 112
            __M_writer(u'<script>\n\n/*\n * \'typ\'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\n\nfunction remote_get_enroll_params(){\n\tvar params ={};\n\n    //params[\'serial\'] =  create_serial(\'LSRE\');\n    params[\'remote.server\'] \t\t= $(\'#remote_server\').val();\n    params[\'remote.local_checkpin\'] = $(\'#remote_local_checkpin\').val();\n    params[\'remote.serial\'] \t\t= $(\'#remote_serial\').val();\n    params[\'remote.user\'] \t\t\t= $(\'#remote_user\').val();\n    params[\'remote.realm\'] \t\t\t= $(\'#remote_realm\').val();\n    params[\'remote.resConf\'] \t\t= $(\'#remote_resconf\').val();\n    params[\'description\'] \t\t\t= "remote:" + $(\'#remote_server\').val();\n\n    jQuery.extend(params, add_user_data());\n\n\treturn params;\n}\n\n\njQuery.validator.addMethod("remote_server", function(value, element, param){\n    return value.match(param);\n}, "')
            # SOURCE LINE 142
            __M_writer(escape(_('Please enter a valid URL for the privacyIDEA server. It needs to start with http:// or https://')))
            __M_writer(u'");\n\n\n\n$("#form_enroll_token").validate({\n         rules: {\n            remote_server: {\n                required: true,\n                number: false,\n                remote_server: /^(http:\\/\\/|https:\\/\\/)/i\n             }\n         }\n     });\n\n')
            # SOURCE LINE 156

            from privacyidea.lib.config import getFromConfig
            sys_remote_server = ""
            sys_remote_realm = ""
            sys_remote_resConf = ""
            sys_checkpin_local = "selected"
            sys_checkpin_remote = ""
            
            try:
                    sys_remote_server = getFromConfig("remote.server")
                    sys_remote_realm = getFromConfig("remote.realm")
                    sys_remote_resConf = getFromConfig("remote.resConf")
                    sys_remote_local_checkpin = getFromConfig("remote.local_checkpin")
            
                    if sys_remote_local_checkpin == 0:
                            sys_checkpin_local = ""
                            sys_checkpin_remote = "selected"
            except Exception:
                    pass
            
            
            
            __M_locals_builtin_stored = __M_locals_builtin()
            __M_locals.update(__M_dict_builtin([(__M_key, __M_locals_builtin_stored[__M_key]) for __M_key in ['sys_remote_realm','sys_checkpin_local','sys_remote_resConf','sys_checkpin_remote','sys_remote_local_checkpin','sys_remote_server','getFromConfig'] if __M_key in __M_locals_builtin_stored]))
            # SOURCE LINE 176
            __M_writer(u'\n</script>\n\n<p>')
            # SOURCE LINE 179
            __M_writer(escape(_("Here you can define to which privacyIDEA Server the authentication request should be forwarded.")))
            __M_writer(u'</p>\n<p>')
            # SOURCE LINE 180
            __M_writer(escape(_("You can either forward the OTP to a remote serial number or to a remote user.")))
            __M_writer(u'</p>\n<p>')
            # SOURCE LINE 181
            __M_writer(escape(_("If you do not enter a remote serial or a remote user, the request will be forwarded to the remote user with the same username")))
            __M_writer(u'</p>\n<table><tr>\n\t<td><label for="remote_server" title=\'')
            # SOURCE LINE 183
            __M_writer(escape(_("You need to enter the server like \'https://privacyidea.my.domain\'")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 184
            __M_writer(escape(_("remote server")))
            __M_writer(u'</label></td>\n\t<td><input class="required" type="text" name="remote_server" id="remote_server"\n\t\tvalue="')
            # SOURCE LINE 186
            __M_writer(escape(sys_remote_server))
            __M_writer(u'" class="text ui-widget-content ui-corner-all"/></td>\n\t</tr><tr>\n\t<td><label for="remote_local_checkpin" title=\'{_("The PIN can either be verified on this local privacyIDEA server or on the remote privacyIDEA server")}\'>\n\t\t')
            # SOURCE LINE 189
            __M_writer(escape(_("check PIN")))
            __M_writer(u'</label></td>\n\t<td><select name="remote_local_checkpin" id="remote_local_checkpin"\n\t\ttitle=\'')
            # SOURCE LINE 191
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or on the remote privacyIDEA server")))
            __M_writer(u"'>\n\t\t<option ")
            # SOURCE LINE 192
            __M_writer(escape(sys_checkpin_remote))
            __M_writer(u' value=0>')
            __M_writer(escape(_("remotely")))
            __M_writer(u'</option>\n\t\t<option ')
            # SOURCE LINE 193
            __M_writer(escape(sys_checkpin_local))
            __M_writer(u' value=1>')
            __M_writer(escape(_("locally")))
            __M_writer(u'</option>\n\t</select></td>\n\t</tr><tr>\n\t<td><label for="remote_serial">')
            # SOURCE LINE 196
            __M_writer(escape(_("remote serial")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="remote_serial" id="remote_serial" value="" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr><tr>\n\t<td><label for="remote_user">')
            # SOURCE LINE 199
            __M_writer(escape(_("remote user")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="remote_user" id="remote_user" value="" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr><tr>\n\t<td><label for="remote_realm">')
            # SOURCE LINE 202
            __M_writer(escape(_("remote user realm")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="remote_realm" id="remote_realm"\n\t\tvalue="')
            # SOURCE LINE 204
            __M_writer(escape(sys_remote_realm))
            __M_writer(u'" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr><tr>\n\t<td><label for="remote_resconf">')
            # SOURCE LINE 206
            __M_writer(escape(_("remote user useridresolver")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="remote_resconf" id="remote_resconf"\n\t\tvalue="')
            # SOURCE LINE 208
            __M_writer(escape(sys_remote_resConf))
            __M_writer(u'" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr></table>\n\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()


